/******************************************
* config.h - Hard coded configuration info
*
* $Id: config.h,v 1.30 2001/09/12 15:27:14 wcampbel Exp $
******************************************/

#ifndef CONFIG_H
#define CONFIG_H

/* These path options MUST be set */

/* This is the root directory of the Stats distribution, where the etc/
** bin/ and logs/ directories are
**
** WARNING:  If this is not set, Stats WILL NOT start up!
*/

#define DPATH "/home/user/Stats/"

/* You MUST set this or RESTART won't work!!!!
** Set this to the path to the stats binary
*/

#define SPATH "/home/user/Stats/bin/stats"

/* Support fnmatch() - This is used to support masked servers
** eg *.fi *.au *.test.net.  This makes it non portable to some
** systems, Windows Cygwin included
** MOST networks do not need this, so it's disabled by default
** With it disabled, introducing a user on a server behind a mask
** will cause Stats to die.  The message when it dies will tell you
** to enable this option.  If you think that you will run a server
** with a mask, enable it now to prevent your service from dying
** NOTE:  It no longer crashes.  It sends out a message via WALLOPS
**        or GLOBOPS, logs the message in the log files, and gracefully
**        exits the service.
*/

#undef FNMATCH

/* Enable the more secure method of communicating with the service, 
** PRIVMSG user@server :message.  This is quite stable and usable, and is
** now enabled by default.  It does SOMEWHAT increase CPU utilization, but
** not by much
*/

#define SUPPORT_MSG_UH

/* Only allow the secure method of communicating with the service, disallowing
** PRIVMSG nick :message.
*/

#undef DISALLOW_NICK_PRIVMSG

/* This is another new feature... This is for support of the Hybrid 6
** NOTICE/PRIVMSG opers@server feature.  If this is enabled, all messages
** to opers@<my name> will be forwarded on to all logged in users.
** This does increase CPU usage so it is disabled by default
*/

#undef SUPPORT_MSG_OPERS

/* Define this to have the service ignore illegal top level domains
** This is useful on servers that allow spoofs for either opers 
** or users.  Currently, this will just prevent logging of the illegal
** TLD's to the log file.  If there is a valid ICANN approved TLD that is
** not working, please let me know so I can fix it.  
**
** NOTE:  I am not putting the Open Root Servers or the ADNS TLD's into
**        the official distribution.  If you use one or the other and
**        wish to have support, then you will have to create a TLD
**        table.  I could then distribute this TLD table along side the
**        official one.
**
** NOTE:  Your statistics are not changed by this option.  Any unknown
**        tld users are assigned to the 'unknown' TLD, and properly
**        shown in TLDMAP
*/

#undef IGNORE_ILLEGAL_TLD

/* If you wish the Stats service to track server VERSION's, and report them
** on a SERVER request and on the HTML dump, define this.  It will increase
** memory usage, CPU, and bandwidth on the network.  It won't be terribly
** noticeable though, and may be worth the extra
*/

#undef TRACK_VERSION

/* Disabling LAG_CHECK will remove Stats ability to track lag between
** servers on the network.  Disabling it will reduce CPU and Bandwidth
** consumption, but at a cost, less useful information stored by the 
** service.  Prior to this release, Stats ALWAYS sent the PING's out
** to the network.
*/

#define LAG_CHECK

/* On Hybrid 7 (and Hybrid 6 if you have an admin mode patch), this option
** will show opers as A O o (with o most likely never being used)
**
** Idea from Xride's ModeServ for Dal
*/

#define OPER_MODES

/* If you dislike the service being able to JUPE, and were unhappy with
** JUPE being controlled by an entry in the config file, this is for you...
** JUPE must be #define'ed here to even exist in the service.
*/

#undef ENABLE_JUPE

/* Define this to allow any oper to issue the JUPE command.  With it undef'ed
** it will only allow people both opered and logged in to the service
**
** This used to be defined by default (in the GeoStats code).  Now it is
** optionally definable.  
*/

#undef ALL_OPER_JUPE

/* This will enable or disable QUOTE/RAW, just as dangerous
** #define it to have it enabled
*/

#undef ENABLE_QUOTE

/* Enable this option to create a 3/4 second artificial delay between
** sequential PRIVMSG's.  Note that this can and will lag your service.
** This option is ONLY in the conf to support servers with extremely
** strict flood control, enough to rate limit remote users... (Hyb7)
**
** NOTE:  This option shouldn't be necessary after Hybrid-7 beta2
*/

#undef DELAY_PRIVMSG

/* Large network support - The targeted audience of this package
** are small networks (Under 1000 users and 60 servers)
** Enabling this option will enlarge the data tables (and increase
** memory usage).  This is untested, and therefore may cause problems
** I would avoid enabling this until there is further testing
**
** This service will need to be heavily optimized to support a large
** number of users.  The optimization HAS begun, but is far from complete.
*/

#undef LARGE_NET

/* Enable the option that will cause the service to gracefully exit
** after a certain number of collisions in a certain period of time
** This prevents 2 services running with the same nickname from massively
** colliding each other.  Disable this on a centrally controlled network, 
** where issues like this cannot occur.
**
** NOTE:  This does mean that an abusive admin can force the service to 
**        die quite easily.  Tradeoffs.....
*/

#define DIE_ON_MASS_COLLIDE

/* Define this option to enable the Spam service.  I see very little use
** for it, so it's disabled by default.  This is one of the things that
** was part of the base GeoStats distribution, so I don't believe in removing
** it completely, as SOME people may want to use it
*/

#undef SPAMSERV

/* Define this option to enable the Split service.  This is the code that
** was part of my old Split service, now incorporated into the superior
** Stats code.
** This code is now appearing to work with the same functionality as
** the old Split code was.
*/

#undef SPLITSERV

/* Define this option to enable the Statistical service.  This is here to
** disable by people who may want to run a split service only, or have
** a service that has no user interface, only a web dump.  This will ONLY
** prevent the bot from appearing on the network.  Statistics are still
** properly kept.
** This is enabled by default
*/

#define STATSERV

/* I hate having to do this... now the IRCD_TYPE is set at compile
** time.  This means to change the IRCD_TYPE, you must recompile the 
** service
**
** Only enable ONE - There is no checking or sanity checking on this 
** option yet...
**
** Supported types...
** IRCD_HYBRID
** IRCD_HYB6
** IRCD_HYB7
** IRCD_OPN
** IRCD_2_10
**
** Look at the README file for more detailed descriptions of these IRCD
** types.
*/
#define IRCD_HYBRID
/* #define IRCD_HYB6 */
/* #define IRCD_HYB7 */
/* #define IRCD_OPN */
/* #define IRCD_2_10 */

/* STOP STOP */
#ifdef IRCD_2_10
#undef OPER_MODES
#endif

#endif /* CONFIG_H */
