/* Stats - proto.h - Function prototypes
**
** $Id: proto.h,v 1.50 2001/09/13 19:31:00 wcampbel Exp $
**
*/

#ifndef PROTO_H
#define PROTO_H

#include "stats.h"

/* Macros that were in m_stats.h */
#define DecreaseOpers(x)        x->opers--; netinfo.opers--;
#define DecreaseUsers(x)        x->users--; netinfo.users--;
#define DecreaseServers()       netinfo.servers--;

/* Simplification macros */
#define IsPrivmsg()	(settings.flags & FLAG_PRIVMSG)
#define IsLag()		(settings.flags & FLAG_LAG)
#define IsOnlyOpers()	(settings.flags & FLAG_ONLYOPERS)
#define IsHtml()	(settings.flags & FLAG_HTML)
#define IsSetOper()	(settings.flags & FLAG_SETOPER)
#define IsHide()	(settings.flags & FLAG_HIDE)
#define IsFlood()	(settings.flags & FLAG_FLOOD)
#define IsVHost()	(settings.flags & FLAG_VHOST)
#define IsUIsOper()	(u->flags & UFLAG_ISOPER)
#define IsUIsAway()	(u->flags & UFLAG_ISAWAY)
#define IsUFloodSent()	(u->flags & UFLAG_FLOODSENT)
#define IsMIsOn()	(myu->flags & MFLAG_ISON)
#define IsDebug()	(me.cmdflags & CMD_DEBUG)
#define IsNoFork()	(me.cmdflags & CMD_NOFORK)

#ifdef SPLITSERV
#define IsSPPrivmsg()	(sp.flags & SPF_PRIVMSG)
#define IsSPChanmsg()	(sp.flags & SPF_CHANMSG)
#define IsSPEnableText()	(sp.flags & SPF_ENABLE_TEXT)
#define IsSPReport()	(sp.flags & SPF_REPORT)
#endif

/* External variables... */
extern int servsock;

#ifdef SPAMSERV
extern char s_Spam[SPAMLEN];
#endif

extern const char version[];

#ifdef SUPPORT_MSG_UH
#ifdef STATSERV
extern char s_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef SPLITSERV
extern char sp_UH[MAXUSER + MAXHOST + 2];
#endif
extern char a_UH[MAXUSER + MAXHOST + 2];
#endif

#ifdef SUPPORT_MSG_OPERS
extern char s_OPERS[MAXHOST + 7];
#endif

#ifdef SPLITSERV
int is_delayed;
time_t notice;
SpServer* map_head;
SpServer* map_uplink;
SpServer* split_table[MAXSERVER];
SpServer* server_table[MAXSERVER];

/* split-util.c */
void dump_map();
void dump_text(FILE*, SpServer*, int);
void html_map(FILE *, SpServer*, int);
char* dash(int);
char* hdash(int);
void SpFlushServerTree(SpServer *);
void spprivmsg(char *, char *, char *, ...);
void PrintMap(char*, SpServer*, int);
void PrintSplit(char*);
char* dash(int);
void PrintDump(char *);
void Forget(char * origin, char * serv);
void Add(char * origin, char * serv);
void Update(char * origin);
void spprivmsg_list(char *, char *, const char **);

/* split-data.c */
void SpInitTables();
SpServer* SpNewServer(char *, char *);
SpServer* SpFindServer(char *);
void SpDelServer(char *);
void SpNewSplit(char *);
void SpDelSplit(char *);
SpServer* SpFindSplit(char *);

/* splitserv.c */
void splitserv(char *, char *, char *);

#endif

/* adminserv.c */
void adminserv(char *, char *);

/* sock.c */
int ConnectTo(char *, int);
void read_loop();
void sts(char *, ...);

/* util.c */
void log(char *, ...);
void splog(char *, ...);
void ResetLogs();
char *sctime(time_t);
char *sftime(time_t);
char *irc_time(time_t);
#if defined(CRYPT) || defined(MD5)
char *make_salt();
#endif /* CRYPT || MD5 */
int tokenize(char *message, char **parv);
int msgtoken(char *message, char **parv);
void *smalloc(size_t);
unsigned long HASH(const unsigned char *, int);
char *strlower(char *);
char *VersionFlags();
#ifdef OPER_MODES
char opermode(User *);
#endif
void SigTerm(int);
int is_channel(char *);

/* conf.c */
void strip(char *);
void InitDefaults();
void ConfLoad();
int GetChanList();
void rehash();

/* main.c */
void login();
#ifdef STATSERV
void init_statserv();
#endif
#ifdef SPAMSERV
void init_spam();
#endif
#ifdef STATSERV
void init_join();
#endif
void init_admin();
#ifdef SPLITSERV
void init_splitserv();
void init_spjoin();
#endif

/* ircd.c */
void parse();
void privmsg(char *, const char *, char *, ...);
void privmsg_list(char *, char *, const char **);
void wallops(char *, char *, ...);
int flood(User *);
void SendQuit(User *, int, char *);
void Trace(User *);
void ParseStats(User *, char);

/* timer.c */
void chk();
void TimerReset();
void TimerSpam();
void TimerPings();
void TimerMidnight();
int is_midnight();

/* statserv.c */
#ifdef STATSERV
void statserv(char *, char *);
#endif

/* spam.c */
#ifdef SPAMSERV
void spam(char *, char *);
#endif

/* users.c */
Server *serverlist[S_TABLE_SIZE];
User *userlist[U_TABLE_SIZE];
MyUser *myuhead;
#ifdef IRCD_2_10
Service *servicehead;
#endif
#ifdef IRCD_2_10
void AddUser(char *, char *, char *, Server *);
#else
void AddUser(char *, char *, char *, char *);
#endif
void DelUser(char *);
void Change_User(User *, char *);
User *finduser(char *);
void UserMode(char *, char *);
void init_user_hash();
#ifdef IRCD_2_10
void AddServer(char *, int, int, char *, char *);
Server *findservtoken(int);
#else
void AddServer(char *, int, char *, char *);
#endif
void FlushServer(char *);
void DelServer(char *);
Server *findserver(char *);
void init_server_hash();
void LoadMyUsers();
void SaveMyUsers();
void DeleteMyUser(char *);
MyUser *findmyuser(char *);
#ifdef IRCD_2_10
Service *findservice(char *, char *);
void AddService(char *, char *, char *, char *);
void DelService(char *, char *);
void DelServicesByServer(char *);
#endif

/* tld.c */
TLD *tldlist[T_TABLE_SIZE];
TLD *findtld(char *);
TLD *AddTLD(User *);
void LoadTLD();
void init_tld();

/* stats.c */
Stats *shead;
char *timediff(time_t);
void IncreaseOpers(Server *);
void IncreaseUsers(Server *);
void IncreaseServers();
void AddStats(Server *);
Stats *findstats(char *);
void SaveStats();
void LoadStats();

/* html.c */
void DumpHTML();

#endif
