/* Stats - struct.h - Structure definitions
**
** $Id: struct.h,v 1.33 2001/09/13 19:31:01 wcampbel Exp $
**
*/

#ifndef STRUCT_H
#define STRUCT_H

#include "stats.h"

/* Our data structures are recursive */
typedef struct server_ Server;
typedef struct user_ User;
typedef struct stats_ Stats;
typedef struct tld_ TLD;
typedef struct myuser_ MyUser;
#ifdef IRCD_2_10
typedef struct service_ Service;
#endif

/* The old bit fields are a gcc feature...replace it with bit flags
** and check with & and |
*/
/* Nothing 0x00000001 */
#define FLAG_PRIVMSG	0x00000002
#define FLAG_LAG	0x00000004
#define FLAG_ONLYOPERS	0x00000008
#define FLAG_HTML	0x00000010
#define FLAG_SETOPER	0x00000020
/* 0x00000040 is unused */
#define FLAG_HIDE	0x00000080
#define FLAG_FLOOD	0x00000100
#define FLAG_VHOST	0x00000200

#define UFLAG_FLOODSENT	0x00000001
#define UFLAG_ISOPER	0x00000002
#define UFLAG_ISAWAY	0x00000004

#define MFLAG_ISON	0x00000001

#define CMD_DEBUG       0x00000001   
#define CMD_NOFORK      0x00000002   

#ifdef OPER_MODES
#define OPER_LOCAL	0x00000001
#define OPER_GLOBAL	0x00000002
#define OPER_ADMIN	0x00000004
#endif

#ifdef SPLITSERV
#define SPF_PRIVMSG	0x00000001
#define SPF_CHANMSG	0x00000002
#define SPF_ENABLE_TEXT	0x00000004
#define SPF_REPORT	0x00000008
#endif

/* This struct is getting out of hand...
** It needs to be split into at least a netinfo and
** a settings struct.  The remaining things will be 
** in the "catch all" me struct
*/
/* you want it? you got it. -screwedup */

struct netinfo
{
	long inb;
	long outb;
        int opers;     
        int maxopers;
        long users;
        long maxusers;
        int servers;
        int maxservers;
        time_t t_maxopers;  
        time_t t_maxusers;  
        time_t t_maxservers;
        unsigned int away; 
        int chan_count;
        int requests;
#ifdef DIE_ON_MASS_COLLIDE
        int collides;
#endif
} netinfo;

struct settings
{
        char name[MAXHOST + 1];
        int port;
        unsigned int r_time;
        char uplink[MAXHOST + 1];
        char vhost[MAXHOST + 1];
        char pass[MAXPASS + 1];
        char services_name[MAXHOST + 1];   
        char desc[LINE + 1];
        unsigned int flags;		
        int lag_time;
        char html[BUFSIZE + 1];  
        int html_time;
        char join_chans[BUFSIZE + 1];
        char url[BUFSIZE + 1];
        int flood;
        int flood_time;
        char netname[MAXNICK + 1];  
        char admin_name[LINE + 1];
        char admin_email[LINE + 1];
} settings;	

struct me
{
	time_t t_start;
	Server *s;
	char *chan_list[MAXCHANS];
	unsigned int cmdflags;
} me;

#ifdef STATSERV
struct StatServ
{
	char nick[MAXNICK + 1];
	char user[MAXUSER + 1];
	char host[MAXHOST + 1];
	char rname[LINE + 1];
} StatServ;
#endif

struct server_
{
	Server *next, *prev;
	char name[MAXHOST + 1];
	char desc[LINE + 1];
#ifdef TRACK_VERSION
	char version[BUFSIZE + 1];
#endif
	int hops;
#ifdef IRCD_2_10
	int token;
#endif
	long hash;
	Stats *stats;
	unsigned int users;
	int opers;
	time_t connected_since;
	time_t last_announce;
#ifdef LAG_CHECK
	long ping;
	long lowest_ping;
	long highest_ping;
	time_t t_lowest_ping;
	time_t t_highest_ping;
#endif
	Server *uplink;
};

struct user_
{
	User *next, *prev;
	char nick[MAXNICK + 1];
	char hostname[MAXHOST + 1];
	char username[MAXUSER + 1];
	Server *server;
	TLD *tld;
	MyUser *myuser;
	int flood;
	time_t t_flood;
	long hash;
#ifdef OPER_MODES
	unsigned int operflags;
#endif
	unsigned int flags;
};

#ifdef IRCD_2_10
struct service_
{
	Service *next, *prev;
	char nick[MAXNICK + 1];
        char server[MAXHOST + 1];
        char dist[MAXHOST + 1];
	char gecos[LINE + 1];
};
#endif

struct stats_
{
	Stats *next;
	char name[MAXHOST + 1];
	int numsplits;
	long maxusers;
	time_t t_maxusers;
	int maxopers;
	time_t t_maxopers;
	time_t lastseen;
	time_t starttime;
	unsigned int operkills;
	unsigned int serverkills;
};

#ifdef LAG_CHECK
struct ping
{
    time_t last_sent;
    long ulag;
} ping;
#endif

struct daily
{
	int servers;
	time_t t_servers;
	long users;
	time_t t_users;
	int opers;
	time_t t_opers;
} daily;

struct tld_
{
	TLD *next;
	long hash;
	char tld[MAXTLD + 1];
	char country[LINE + 1];
	int users;
};

struct myuser_
{
	MyUser *next;
	char username[MAXNICK + 1];
	char password[LINE + 1];
	time_t lastseen;
	User *u;
	unsigned int flags;
};

struct MsgTable
{
        char* name;
	long counter;
        int (*handler)();
};

#ifdef SPLITSERV
typedef struct spserver_ SpServer;

struct spserver_
{
        char name[MAXHOST + 1];
        SpServer* uplink;
        int numleaf;
        time_t ts;
        SpServer* leaflist[MAXSERVER];
};

struct SplitServ
{
        char host[MAXHOST + 1];
        char user[MAXUSER + 1];
        char nick[MAXNICK + 1];
} SplitServ;

struct sp
{
        char uplink[MAXHOST + 1];
        char desc[LINE + 1];
        char version[LINE + 1];
        int numserver;
        char join_chan[LINE + 1];
        int delay;
        int text_time;
	unsigned int flags;
        char text_name[BUFSIZE + 1];
} sp;
#endif

struct AdminServ
{
        char host[MAXHOST + 1];
        char user[MAXUSER + 1];
        char nick[MAXNICK + 1];
} AdminServ;

#endif
