/* Stats - adminserv.c - Administration Service Functionality
** $Id: adminserv.c,v 1.16 2001/09/12 15:27:17 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "help.h"
#include "patchlevel.h"

#if defined(DEBIAN) || defined(CYGWIN) || defined(SOLARIS)
#include <crypt.h>
#endif

#if defined(LINUX) || defined(REDHAT)
/* A kludge in mkpasswd.c, since modern Linux systems require _XOPEN_SOURCE
** to be #define'ed BUT... it screws up the rest of the source, preventing
** all str* functions from being defined
*/
extern char *crypt();
#endif

extern const char version_date[], version_time[];
static char errstr[BUFSIZE + 1];

int a_login(char *, int, char **);
int a_user(char *, int, char **);
int a_shutdown(char *, int, char **);
int a_restart(char *, int, char **);
#ifdef ENABLE_JUPE
int a_jupe(char *, int, char **);
#endif
int a_setpass(char *, int, char **);
#ifdef ENABLE_QUOTE
int a_quote(char *, int, char **);
#endif
int a_cping(char *, int, char **);
int a_cversion(char *, int, char **);
int a_help(char *, int, char **);

static struct MsgTable admin_table[] =
{
	{ "HELP",	0,	a_help },
	{ "LOGIN",	0,	a_login },
	{ "USER",	0,	a_user },
	{ "DIE",	0,	a_shutdown },
	{ "SHUTDOWN",	0,	a_shutdown },
	{ "RESTART",	0,	a_restart },
#ifdef ENABLE_JUPE
	{ "JUPE",	0,	a_jupe },
#endif
	{ "SETPASS",	0,	a_setpass },
#ifdef ENABLE_QUOTE
	{ "QUOTE",	0,	a_quote },
	{ "RAW",	0,	a_quote },
#endif
	{ "\001PING",	0,	a_cping },
	{ "\001VERSION\001",	0,	a_cversion },
	{ (char *)0,	0,	(int (*)()) 0 }
};

static struct HelpTable help_table[] = {
        { "LOGIN",      as_login_help,          0 },
        { "USER",       as_user_help,           1 },
        { "SHUTDOWN",   as_shutdown_help,       1 },
        { "RESTART",    as_restart_help,        1 },
        { "SETPASS",    as_setpass_help,        1 },
#ifdef ENABLE_JUPE
        { "JUPE",       as_jupe_help,           0 },
#endif
#ifdef ENABLE_QUOTE
        { "QUOTE",      as_quote_help,          1 },
        { "RAW",        as_quote_help,          1 },
#endif
        { (char *)0,    (const char **)0,       0 }
};


void adminserv(char *origin, char *param)
{
	char *parv[MAXPARA];
	int parc = 0;
	User *u = NULL;
	int i;
	int status;
	static char buf[BUFSIZE + 1];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* Clear the error string */
	errstr[0] = '\0';

	strncpy(buf, param, BUFSIZE);
	buf[BUFSIZE] = '\0';

	if (param)
		parc = msgtoken(param, parv);

	if (origin)
		u = finduser(origin);
	else
		return;

        if (!u)
	{
                log("Unable to finduser %s (adminserv)", origin);
                return;
        }

	netinfo.requests++;

	if (!parc)
		return;

	if (parv[0][0] != '\001')
		log("%s received message from %s: %s", AdminServ.nick, 
			origin, buf);

        if (IsOnlyOpers() && !IsUIsOper())
	{
		if (parv[0][0] != '\001')
		{
                	privmsg(u->nick, AdminServ.nick,
                       		 "This service is only available to IRCops.");
                	return;
		}
        }

	/* All the preliminary junk is done, now we have to go through 
	** the tables and call the handler for that command
	*/
        for (i=0; admin_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], admin_table[i].name))
                {
                    status = admin_table[i].handler(u->nick, parc, parv);
                    if (status == 1)
                        log("Error with adminserv command %s: %s", parv[0],
			(errstr) ? errstr : "No error message");
		    /* Matched a value, done, then return */
		    return;
                }
        }
	/* If we get here, we did not get a valid command...
	** Make sure not to say anything to the user if it was a 
	** CTCP that was not matched
	*/
	if (*parv[0] == '\001')
		return;
	privmsg(u->nick, AdminServ.nick, "Invalid command: %s", parv[0]);
	return;
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

int a_help(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	int i;

	if (!parv[1])
	{
		privmsg_list(u->nick, AdminServ.nick, as_help);
		if (u->myuser != NULL)
			privmsg_list(u->nick, AdminServ.nick, as_myuser_help);
		return 0;
	}

	for (i = 0; help_table[i].helpmsg; i++)
	{
		if (!strcasecmp(parv[1], help_table[i].name))
                {
			if (help_table[i].myuonly)
			{
				if (u->myuser)
					privmsg_list(u->nick, AdminServ.nick,
						help_table[i].helpmsg);
				else
					privmsg(u->nick, AdminServ.nick,
						"Permission Denied.");
			}
			else
			{
				privmsg_list(u->nick, AdminServ.nick,
					help_table[i].helpmsg);
			}

			return 0;
		}
	}

	privmsg(u->nick, AdminServ.nick, "Unknown Help Topic: \2%s\2", parv[1]);
	snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
	return 1;
}

int a_login(char *origin, int parc, char *parv[])
{
        MyUser *myu;
#if defined(CRYPT)
        char * crpass;
#endif
	char *user=parv[1];
	char *pass=parv[2];
	User *u = finduser(origin);

        if (!user || !pass)
	{
                privmsg(origin, AdminServ.nick,
			"Syntax: /msg %s LOGIN \37username\37 \37password\37",
			AdminServ.nick);
		strcpy(errstr, "Missing parameters in LOGIN");
                return 1;
        }

	if (!u)
	{
		snprintf(errstr, BUFSIZE, "Unknown user: %s", origin);
		return 1;
	}

        myu = findmyuser(user);
        if (!myu)
	{
                privmsg(origin, AdminServ.nick, "Unknown User.");
                log("Failed login attempt (unknown user (%s))", user);
		/* This isn't an error in the code, only to the user */
                return 0;
        }
#if defined(CRYPT)
        /* Passing in the whole password is fine, no need to rip of the
        ** salt.  This actually will support md5 without changes here
        */
        crpass = crypt(pass, myu->password);
        if (!strcasecmp(myu->username, user) && 
            !strcmp(myu->password, crpass))
#else
        if (!strcasecmp(myu->username, user) && 
            !strcmp(myu->password, pass))
#endif /* CRYPT */
	{
                log("%s [%s](%s@%s) sucessfully logged in.", u->nick,
                        myu->username, u->username, u->hostname);
                u->myuser = myu;
                privmsg(u->nick, AdminServ.nick, "Permission Granted.");
                myu->lastseen = time(NULL);
                myu->flags |= MFLAG_ISON;
                /* We keep the CURRENT person who LOGIN'ed through that
                ** access entry pointed to by the u pointer.  This is used
                ** to obtain information on the user who logged in, ie.
                ** in STATS p
                */
                myu->u = u;
                return 0;
        }
	else
	{
                log("%s [%s](%s) failed login.", u->nick, myu->username,
			u->hostname);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* This isn't a code error either */
		return 0;
        }

	/* NOT REACHED */
	return 0;
}

int a_user(char *origin, int parc, char *parv[])
{
        int a = 0;
	User *u = finduser(origin);
	char *what = parv[1];
	char *user = parv[2];
	char *pass = parv[3];

        if (!what)
	{
                privmsg(u->nick, AdminServ.nick,
			"Syntax: /msg %s USER [ADD|DEL|LIST]", AdminServ.nick);
                privmsg(u->nick, AdminServ.nick,
			"For additional help, /msg %s HELP", AdminServ.nick);
		strcpy(errstr, "Missing parameter in USER");
                return 1;
        }

        if (!u->myuser)
	{
                log("Denied access (USER) to %s [%s].", u->nick, u->hostname);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!strcasecmp(what, "LIST"))
                a = 0;
        else if (!strcasecmp(what, "ADD"))
                a = 1;
        else if (!strcasecmp(what, "DEL"))
                a = 2;
        else
	{
                privmsg(u->nick, AdminServ.nick, "Unknown Flag: %s", what);
		/* Not REALLY a code error...but... */
		snprintf(errstr, BUFSIZE, "Unknown flag: %s in USER", what);
                return 1;
        }

        switch (a)
	{
                case 0:
		{
                        MyUser *myu;
                        privmsg(u->nick, AdminServ.nick, "UserList:");
                        for (myu = myuhead; myu; myu = myu->next) {
                                privmsg(u->nick, AdminServ.nick, 
					"%-20s (%s) -> %s", myu->username,
                                        (!IsMIsOn()) ? 
						"offline" : " online",
                                        (myu->lastseen == 0) ? "Never Seen" :
						sftime(myu->lastseen));
                        }
                        privmsg(u->nick, AdminServ.nick, "End of List.");
			break;
		}
                case 1:
		{
                        MyUser *myu;
#if defined(CRYPT)
                        char * salt;
#endif /* CRYPT */
                        if (!user || !pass)
			{
                                privmsg(u->nick, AdminServ.nick,
					"Syntax: /msg %s USER ADD "
                                        "<username> <password>",
					AdminServ.nick);
				strcpy(errstr, "Missing parameter in USER");
                                return 1;
                        }
                        myu = smalloc(sizeof(MyUser));
                        strncpy(myu->username, user, MAXNICK);
#if defined(CRYPT)
                        salt = make_salt();
                        strncpy(myu->password, crypt(pass, salt), LINE);
#else
                        strncpy(myu->password, pass, LINE);
#endif /* CRYPT */
                        myu->lastseen = time(NULL);
                        if (!myuhead)
			{
                                myuhead = myu;
                                myuhead->next = NULL;
                        }
			else
			{
                                myu->next = myuhead;
                                myuhead = myu;
                        }
                        privmsg(u->nick, AdminServ.nick,
				"Added user %s with password %s", user, pass);
                        log("%s [%s](%s) added user %s", u->nick,
				u->myuser->username, u->hostname,
				myu->username);
                        SaveMyUsers();
			return 0;
			/* NOT REACHED */
			break;
		}
                case 2:
		{
                        if (!user)
			{
                                privmsg(u->nick, AdminServ.nick,
					"Syntax: /msg %s USER DEL <username>",
                                        AdminServ.nick);
				strcpy(errstr, "Missing parameter in USER");
				return 1;
                        }
                        if ((findmyuser(user)) == NULL)
			{
                                privmsg(u->nick, AdminServ.nick,
					"Unable to find user %s", user);
				snprintf(errstr, BUFSIZE,
					"Unknown myuser: %s", user);
				return 1;
                        }
                        DeleteMyUser(user);
                        SaveMyUsers();
                        privmsg(u->nick, AdminServ.nick, "Deleted user %s",
				user);
                        log("%s [%s](%s) deleted user %s", u->nick,
				u->myuser->username, u->hostname, user);
			return 0;
			break;
		}
        }

	/* NOT REACHED */
	return 0;
}

int a_shutdown(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char comm[BUFSIZE + 1];
	int i;

        if (!u->myuser)
	{
                log("Access Denied (SHUTDOWN) to %s", u->nick);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

	comm[0]='\0';

        if (parv[1])
                strncpy(comm, parv[1], BUFSIZE);
        for (i = 2; i < parc; i++)
                snprintf(comm, BUFSIZE, "%s %s", comm, parv[i]);

	comm[BUFSIZE] = '\0';

        /* Yes, we are very nasty like this. */
        wallops(settings.name, "%s requested SHUTDOWN: %s", u->nick,
                (comm[0]) ? comm : "No Reason");
        SendQuit(u, 0, comm);
        sleep(1);
        close(servsock);
        SaveStats();
        log("%s [%s](%s) requested SHUTDOWN: %s", u->nick, u->myuser->username,
                u->hostname, (comm) ? comm : "No Reason");
        exit(0);

	/* NOT REACHED */
	return 0;
}

int a_restart(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char comm[BUFSIZE + 1];
	int i;

        if (!u->myuser)
	{
                log("Access Denied (RESTART) to %s", u->nick);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

	comm[0]='\0';

	if (parv[1])
		strncpy(comm, parv[1], BUFSIZE);
	for (i = 2; i < parc; i++)
		snprintf(comm, BUFSIZE, "%s %s", comm, parv[i]);

	comm[BUFSIZE] = '\0';

        /* Yes, we are very nasty like this. */
        wallops(settings.name, "%s requested RESTART: %s", u->nick,
                (comm[0]) ? comm : "No Reason");
        SendQuit(u, 1, comm);
        sleep(1);
        close(servsock);
        SaveStats();
        log("%s [%s](%s) requested RESTART: %s", u->nick, u->myuser->username,
                u->hostname, (comm) ? comm : "No Reason");
        execv(SPATH, NULL);

	/* NOT REACHED */
	return 0;
}

#ifdef ENABLE_QUOTE
int a_quote(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char command[BUFSIZE + 1];
	int i;

        if (!u->myuser)
	{
                log("Access Denied (QUOTE) to %s", u->nick);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not an error in the code */
                return 0;
        }

        if (parv[1])
                strncpy(command, parv[1], BUFSIZE);
        for (i = 2; i < parc; i++)
                snprintf(command, BUFSIZE, "%s %s", command, parv[i]);

	command[BUFSIZE] = '\0';

        if (!command)
	{
                privmsg(u->nick, AdminServ.nick,
			"Please enter a valid command");
		strcpy(errstr, "NULL command attempt in QUOTE");
                return 1;
        }

        sts(command);
        log("%s!%s@%s made me execute %s", u->nick, u->username,
                u->hostname, command);

	return 0;
}
#endif

#ifdef ENABLE_JUPE
int a_jupe(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *server = parv[1];
        Server *s;
        /* NOTE:  should I prevent the main services server from
        ** being juped here?
        ** Also prevent my uplink from being JUPED.
        ** JUPE is a touchy feature anyway...
        **
        ** ALSO.... We should maybe make JUPE's more perminant...
        ** Or just remove JUPE functionality completely and use
        ** an updated jupe.c/jupe.pl or main services.
        */

#ifdef ALL_OPER_JUPE
        if (!IsUIsOper())
#else
        if (!IsUIsOper() || !u->myuser)
#endif
	{
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }
        if (!server)
	{
                privmsg(u->nick, AdminServ.nick,
                        "Syntax: /msg %s JUPE <server>",
			AdminServ.nick);
		strcpy(errstr, "Missing parameter in JUPE");
                return 1;
        }
        s = findserver(server);
        if (s)
	{
                sts(":%s SQUIT %s :JUPED by %s", AdminServ.nick, server,
                        u->nick);
                wallops("SERVER %s JUPED by %s", server, u->nick);
                log("SERVER %s SQUIT'ed by JUPE", server);
        }
        sts(":%s SERVER %s 1 :[Jupitered] by %s", settings.name, server,
                u->nick);
        log("%s!%s@%s jupitered %s", u->nick, u->username,
                u->hostname, server);

	return 0;
}
#endif

int a_setpass(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *pass = parv[1];
#if defined(CRYPT)
        char * salt;
#endif /* CRYPT */

        if (!u->myuser)
	{
                log("Access Denied (SETPASS) to %s", u->nick);
                privmsg(u->nick, AdminServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!pass)
	{
                privmsg(u->nick, AdminServ.nick,
			"Please enter a valid password");
		strcpy(errstr, "NULL password attempt in SETPASS");
                return 1;
        }

#if defined(CRYPT)
        salt = make_salt();
        strncpy(u->myuser->password, crypt(pass, salt), LINE);
#else
        strncpy(u->myuser->password, pass, LINE);
#endif /* CRYPT */

        privmsg(u->nick, AdminServ.nick, "Password changed to %s", pass);
        /* No I will NOT log the password! */
        log("%s!%s@%s changed his/her password", u->nick, u->username,
                u->hostname);
	SaveMyUsers();

	return 0;
}

int a_cping(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *command = parv[1];

        if (!command)
                return 1;
        sts(":%s NOTICE %s :\001PING %s", AdminServ.nick, u->nick, command);
        log("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

int a_cversion(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        sts(":%s NOTICE %s :\001VERSION %s(%s) %s %s -> %s %s\001",
	     AdminServ.nick, u->nick, PATCHLEVEL, SERIALNUM, settings.name,
	     VersionFlags(), version_date, version_time);
        log("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

