/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: conf.c,v 1.26 2001/09/13 19:31:03 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

struct ConfTable
{
	char *name;
	void (*handler)();
};

static void c_server_name(char *);
static void c_server_port(char *);
static void c_connect_to(char *);
static void c_connect_pass(char *);
static void c_use_vhost(char *);
static void c_vhost(char *);
static void c_server_desc(char *);
static void c_netname(char *);
static void c_admin_name(char *);
static void c_admin_email(char *);
static void c_reconnect_time(char *);
static void c_flood(char *);
static void c_flood_time(char *);
static void c_flood_ignore(char *);
static void c_lag_time(char *);
#ifdef STATSERV
static void c_statserv_user(char *);
static void c_statserv_host(char *);
static void c_statserv_nick(char *);
#endif
static void c_enable_spam(char *);
static void c_onlyopers(char *);
static void c_want_privmsg(char *);
static void c_services_name(char *);
static void c_lag_notices(char *);
static void c_enable_html(char *);
static void c_set_oper(char *);
static void c_hide_services(char *);
static void c_html_dump(char *);
static void c_html_time(char *);
static void c_ircd_type(char *);
static void c_channels(char *);
static void c_url(char *);
#ifdef SPLITSERV
static void c_splitserv_desc(char *);
static void c_splitserv_delay(char *);
static void c_splitserv_host(char *);
static void c_splitserv_user(char *);
static void c_splitserv_nick(char *);
static void c_splitserv_host(char *);
static void c_splitserv_privmsg(char *);
static void c_splitserv_chanmsg(char *);
static void c_splitserv_enable_text(char *);
static void c_splitserv_report(char *);
static void c_splitserv_text_time(char *);
static void c_splitserv_text_name(char *);
static void c_splitserv_chan(char *);
#endif
static void c_adminserv_host(char *);
static void c_adminserv_nick(char *);
static void c_adminserv_user(char *);

static struct ConfTable conf_table[] =
{
	{ "SERVER_NAME",	c_server_name },
	{ "SERVER_PORT",	c_server_port },
	{ "CONNECT_TO",		c_connect_to },
	{ "CONNECT_PASS",	c_connect_pass },
	{ "USE_VHOST",		c_use_vhost },
	{ "VHOST",		c_vhost },
	{ "SERVER_DESC",	c_server_desc },
	{ "NETNAME",		c_netname },
	{ "ADMIN_NAME",		c_admin_name },
	{ "ADMIN_EMAIL",	c_admin_email },
	{ "RECONNECT_TIME",	c_reconnect_time },
	{ "FLOOD_TIME",		c_flood_time },
	{ "FLOOD",		c_flood },
	{ "FLOOD_IGNORE",	c_flood_ignore },
	{ "LAG_TIME",		c_lag_time },
#ifdef STATSERV
	{ "STATSERV_HOST",	c_statserv_host },
	{ "STATSERV_USER",	c_statserv_user },
	{ "STATSERV_NICK",	c_statserv_nick },
#endif
	{ "ENABLE_SPAM",	c_enable_spam },
	{ "ONLYOPERS",		c_onlyopers },
	{ "WANT_PRIVMSG",	c_want_privmsg },
	{ "SERVICES_NAME",	c_services_name },
	{ "LAG_NOTICES",	c_lag_notices },
	{ "ENABLE_HTML",	c_enable_html },
	{ "SET_OPER",		c_set_oper },
	{ "HIDE_SERVICES",	c_hide_services },
	{ "HTML_DUMP",		c_html_dump },
	{ "HTML_TIME",		c_html_time },
	{ "IRCD_TYPE",		c_ircd_type },
	{ "CHANNELS",		c_channels },
	{ "URL",		c_url },
#ifdef SPLITSERV
	{ "SPLITSERV_DESC",	c_splitserv_desc },
	{ "SPLITSERV_DELAY",	c_splitserv_delay },
	{ "SPLITSERV_HOST",	c_splitserv_host },
	{ "SPLITSERV_USER",	c_splitserv_user },
	{ "SPLITSERV_NICK",	c_splitserv_nick },
	{ "SPLITSERV_PRIVMSG",	c_splitserv_privmsg },
	{ "SPLITSERV_CHANMSG",	c_splitserv_chanmsg },
	{ "SPLITSERV_ENABLE_TEXT",	c_splitserv_enable_text },
	{ "SPLITSERV_REPORT",	c_splitserv_report },
	{ "SPLITSERV_TEXT_TIME",c_splitserv_text_time },
	{ "SPLITSERV_TEXT_NAME",c_splitserv_text_name },
	{ "SPLITSERV_CHAN",	c_splitserv_chan },
#endif
	{ "ADMINSERV_USER",	c_adminserv_user },
	{ "ADMINSERV_HOST",	c_adminserv_host },
	{ "ADMINSERV_NICK",	c_adminserv_nick },
	{ (char *) 0,		(void (*)()) 0 }
};

/* Remove CR and LF from the line input.  ASSUME that the line 
** does NOT contain CR or LF in the middle
*/
void strip(char *line)
{
	char *c;

	if ((c = strchr(line, '\r')))
	{
		/* If we receive a CR, then we can just terminate the
		** call there.
		*/
		*c = '\0';
		return;
	}

	if ((c = strchr(line, '\n')))
	{
		*c = '\0';
		return;
	}

	return;
}

/* Set default parameters.  The settings used are the same as
** what is provided in stats.cfg.dist except for the binary
** ones, which MUST be default 0 in order for the conf load
** to work properly
*/
void InitDefaults()
{
	/* These strcpy's are safe, as they are static strings */

	/* All bit values must be initialized to 0 */
	strcpy(settings.name, "stats.network.tld");
	settings.port = 6666;
	strcpy(settings.uplink, "hub.network.tld");
	strcpy(settings.vhost, "0.0.0.0");
	strcpy(settings.pass, "password");
	strcpy(settings.desc, "Network Statistical Services");
	settings.flags=0x00000000; /* 0 is fine, but be explicit */
	settings.r_time = 10;
	settings.flood = 6;
	settings.flood_time = 10;
#ifdef STATSERV
	strcpy(StatServ.host, "stats.network.tld");
	strcpy(StatServ.user, "stats");
	strcpy(StatServ.nick, "StatServ");
#endif
	strcpy(settings.services_name, "services.network.tld");
	settings.lag_time = 45;
	strcpy(settings.html, "/home/user/public_html/stats.html");
	settings.html_time = 3600;
	strcpy(settings.join_chans, "#statserv,#opers");
	strcpy(settings.url, "http://www.network.tld/stats.html");
	strcpy(settings.netname, "Misconfigured");
	strcpy(settings.admin_name, "Misconfigured Admin (mis-nick)");
	strcpy(settings.admin_email, "admin@misconfigured.net");
	/* The chan_list is derived from the join_chans...don't set it */
#ifdef SPLITSERV
	strcpy(sp.desc, "Split Services");
	sp.delay = 15;
	strcpy(SplitServ.host, "split.network.tld");
	strcpy(SplitServ.user, "split");
	strcpy(SplitServ.nick, "SplitServ");
	sp.flags = 0;
	sp.text_time = 1800;
	strcpy(sp.text_name, "map.txt");
	strcpy(sp.join_chan, "#split");
#endif
	strcpy(AdminServ.nick, "AdminServ");
	strcpy(AdminServ.user, "admin");
	strcpy(AdminServ.host, "admin.network.tld");
}

void ConfLoad()
{
        FILE *fp;
        char buf[BUFSIZE];
	char *cmd = buf;
        char *param;
	int i, found;

        if ((fp = fopen(CONFIG_NAME, "r")) == NULL)
        {
                printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
                exit(0);
        }

        while (fgets(buf, BUFSIZE, fp))
	{
		strip(buf);
		if (!buf[0])
			continue;
                if (buf[0] == '#')
                        continue;
		param = strchr(buf, ' ');
		if (param)
		{
			*param = '\0';
			param++;
		}
		else
		{
			param = NULL;
		}
		found = 0;
		
		for (i=0; conf_table[i].name; i++)
		{
			/* using strncasecmp will be bad here, consider
			** "FLOOD"
			*/
			if (!strcasecmp(cmd, conf_table[i].name))
			{
				conf_table[i].handler(param);
				found = 1;
				break;
			}
		}

		if (!found)
			log("Invalid conf file option: %s", cmd);
	}

	fclose(fp);
}

static void c_server_name(char *param)
{
	if (param)
	{
		memcpy(settings.name, param, sizeof(settings.name));
		settings.name[MAXHOST - 1] = '\0';
	}
	else
	{
		log("Invalid parameter for SERVER_NAME");
	}
}

static void c_server_port(char *param)
{
        if (param)
        {
		settings.port = atoi(param);
        }
        else
        {
                log("Invalid parameter for SERVER_PORT");
        }
}

static void c_connect_to(char *param)
{
        if (param)
	{
                memcpy(settings.uplink, param, sizeof(settings.uplink));
                settings.uplink[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for CONNECT_TO");
        }
}

static void c_connect_pass(char *param)
{
        if (param)
	{
                memcpy(settings.pass, param, sizeof(settings.pass));
                settings.pass[MAXPASS - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for CONNECT_PASS");
        }
}

static void c_use_vhost(char *param)
{
	settings.flags |= FLAG_VHOST;
}

static void c_vhost(char *param)
{
        if (param)
	{
                memcpy(settings.vhost, param, sizeof(settings.vhost));
                settings.vhost[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for VHOST");
        }
}

static void c_server_desc(char *param)
{
        if (param)
	{
                memcpy(settings.desc, param, sizeof(settings.desc));
                settings.desc[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SERVER_DESC");
        }
}

static void c_netname(char *param)
{
        if (param)
	{
                memcpy(settings.netname, param, sizeof(settings.netname));
                settings.netname[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for NETNAME");
        }
}

static void c_admin_name(char *param)
{
        if (param)
	{
                memcpy(settings.admin_name, param, sizeof(settings.admin_name));
                settings.admin_name[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMIN_NAME");
        }
}

static void c_admin_email(char *param)
{
        if (param)
	{
                memcpy(settings.admin_email, param,
			sizeof(settings.admin_email));
                settings.admin_email[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMIN_EMAIL");
        }
}

static void c_reconnect_time(char *param)
{
        if (param)
	{
		settings.r_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for RECONNECT_TIME");
        }
}

static void c_flood(char *param)
{
        if (param)
	{
		settings.flood = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD");
        }
}

static void c_flood_time(char *param)
{
        if (param)
	{
		settings.flood_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD_TIME");
        }
}

static void c_flood_ignore(char *param)
{
	settings.flags |= FLAG_FLOOD;
}

static void c_lag_time(char *param)
{
        if (param)
	{
		settings.lag_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for LAG_TIME");
        }
}

#ifdef STATSERV
static void c_statserv_host(char *param)
{
        if (param)
	{
                memcpy(StatServ.host, param, sizeof(StatServ.host));
                StatServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_HOST");
        }
}

static void c_statserv_user(char *param)
{
        if (param)
	{
                memcpy(StatServ.user, param, sizeof(StatServ.user));
                StatServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_USER");
        }
}

static void c_statserv_nick(char *param)
{
        if (param)
	{
                memcpy(StatServ.nick, param, sizeof(StatServ.nick));
                StatServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_NICK");
        }
}
#endif

static void c_enable_spam(char *param)
{
	printf("ENABLE_SPAM is a compile time option.\n");
	printf("Please remove this option from the\n");
	printf("%s file.\nNOT FATAL\n", CONFIG_NAME);
}

static void c_onlyopers(char *param)
{
	settings.flags |= FLAG_ONLYOPERS;
}

static void c_want_privmsg(char *param)
{
	settings.flags |= FLAG_PRIVMSG;
}

static void c_services_name(char *param)
{
        if (param)
        {
                memcpy(settings.services_name, param,
			sizeof(settings.services_name));
                settings.services_name[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SERVICES_NAME");
        }
}

static void c_lag_notices(char *param)
{
	settings.flags |= FLAG_LAG;
}

static void c_enable_html(char *param)
{
	settings.flags |= FLAG_HTML;
}

static void c_set_oper(char *param)
{
	settings.flags |= FLAG_SETOPER;
}

static void c_hide_services(char *param)
{
	settings.flags |= FLAG_HIDE;
}

static void c_html_dump(char *param)
{
        if (param)
        {
                memcpy(settings.html, param, sizeof(settings.html));
                settings.html[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for HTML_DUMP");
        }
}

static void c_html_time(char *param)
{
        if (param)
        {
		settings.html_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for HTML_TIME");
        }
}

static void c_ircd_type(char *param)
{
	printf("The IRCD_TYPE Option has been disabled.\n");
	printf("Please remove the IRCD_TYPE from %s\n", CONFIG_NAME);
	printf("and choose the IRCD_type define in config.h\n");
	printf("\nStats terminated.\n");
	exit(1);
}

static void c_channels(char *param)
{
        if (param)
        {
                memcpy(settings.join_chans, param, sizeof(settings.join_chans));
                settings.join_chans[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for CHANNELS");
        }
}

static void c_url(char *param)
{
        if (param)
        {
                memcpy(settings.url, param, sizeof(settings.url));
                settings.url[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for URL");
        }
}

#ifdef SPLITSERV
static void c_splitserv_desc(char *param)
{
        if (param)
        {
                memcpy(sp.desc, param, sizeof(sp.desc));
                sp.desc[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DESC");
        }
}

static void c_splitserv_delay(char *param)
{
        if (param)
        {
		sp.delay = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DELAY");
        }
}

static void c_splitserv_host(char *param)
{
        if (param)
        {
                memcpy(SplitServ.host, param, sizeof(SplitServ.host));
                SplitServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_HOST");
        }
}

static void c_splitserv_user(char *param)
{
        if (param)
        {
                memcpy(SplitServ.user, param, sizeof(SplitServ.user));
                SplitServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_USER");
        }
}

static void c_splitserv_nick(char *param)
{
        if (param)
        {
                memcpy(SplitServ.nick, param, sizeof(SplitServ.nick));
                SplitServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_NICK");
        }
}

static void c_splitserv_privmsg(char *param)
{
	sp.flags |= SPF_PRIVMSG;
}

static void c_splitserv_chanmsg(char *param)
{
	sp.flags |= SPF_CHANMSG;
}

static void c_splitserv_enable_text(char *param)
{
        sp.flags |= SPF_ENABLE_TEXT;
}

static void c_splitserv_report(char *param)
{
        sp.flags |= SPF_REPORT;
}

static void c_splitserv_text_time(char *param)
{
        if (param)
        {
		sp.text_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_TIME");
        }
}

static void c_splitserv_text_name(char *param)
{
        if (param)
        {
                memcpy(sp.text_name, param, sizeof(sp.text_name));
                sp.text_name[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_NAME");
        }
}

static void c_splitserv_chan(char *param)
{
        if (param)
        {
                memcpy(sp.join_chan, param, sizeof(sp.join_chan));
                sp.join_chan[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_CHAN");
        }
}
#endif

static void c_adminserv_user(char *param)
{
        if (param)
        {
                memcpy(AdminServ.user, param, sizeof(AdminServ.user));
                AdminServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_USER");
        }
}

static void c_adminserv_host(char *param)
{
        if (param)
        {
                memcpy(AdminServ.host, param, sizeof(AdminServ.host));
                AdminServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_HOST");
        }
}

static void c_adminserv_nick(char *param)
{
        if (param)
        {
                memcpy(AdminServ.nick, param, sizeof(AdminServ.nick));
                AdminServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_NICK");
        }
}

int GetChanList()
{
	char *s;
	int count = 0;
	for (s = strtok(settings.join_chans, ","); s; s = strtok(NULL, ","))
	{
		if (count == MAXCHANS)
		{
			log("Maximum number of channels join'ed");
			return count;
		}
		/* I know the ++ can be inside the brackets...it's just less
		** readable that way
		*/
		if (*s == ' ')
			s++;
		me.chan_list[count] = strdup(s);
		count++;
	}

	return count;
}

void rehash()
{
	/* still nothing, yet */

	/* I need to add in a limited conf reloading - mark flags
	** for what has been changed, send notices etc.  Some thing
	** won't be able to dynamically change, either do a disconnect
	** and reconnect or do a restart when needed... or demand the 
	** admin do it...bloody complicated
	*/

	/* Add an option to reload the user file as well - maybe in
	** a seperate function
	*/

	/* Free and reload the TLD file too */

	/* I should probably have dynamic settings too, and will need
	** a file to store dynamic settings in, read after the main 
	** conf file... gets confusing...bah
	*/
}
