/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
*  Main parsing routine and ircd related commands
*
** GeoStats CVS Identification
** $Id: ircd.c,v 1.100 2001/09/12 15:27:18 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "numeric.h"
#include "patchlevel.h"

extern const char version_date[], version_time[];
extern const char* infotext[];

char errstr[BUFSIZE + 1];

void ShowMOTD(char *);
void CollideDie();

#ifdef SUPPORT_MSG_OPERS
void MsgOpers(char *, char *);
#endif

int m_time(char *, int, char **);
int m_version(char *, int, char **);
int m_motd(char *, int, char **);
int m_privmsg(char *, int, char **);
int m_notice(char *, int, char **);
int m_server(char *, int, char **);
int m_squit(char *, int, char **);
int m_quit(char *, int, char **);
#ifdef TRACK_VERSION
int m_351(char *, int, char **);
#endif
int m_mode(char *, int, char **);
int m_kill(char *, int, char **);
#ifdef LAG_CHECK
int m_pong(char *, int, char **);
#endif
#if defined(IRCD_HYB7) || defined(IRCD_OPN)
int m_away(char *, int, char **);
#endif
int m_nick(char *, int, char **);
#ifdef IRCD_2_10
int m_service(char *, int, char **);
#endif
int m_ping(char *, int, char **);
int m_admin(char *, int, char **);
int m_stats(char *, int, char **);
int m_trace(char *, int, char **);
int m_pass(char *, int, char **);
int m_info(char *, int, char **);
int m_whois(char *, int, char **);
#ifdef SPLITSERV
int m_invite(char *, int, char **);
#endif

static struct MsgTable msg_table[] =
{
#ifdef LAG_CHECK
	/* PONG is the most used function */
        { "PONG",       (long) 0,       m_pong },
#endif
	{ "PING",       (long) 0,       m_ping },
	/* Depending on how many and how busy the channels that
	** the service sits in are, this can be a heavily used
	** function
	*/
	{ "PRIVMSG",    (long) 0,       m_privmsg },
	/* These are all used often, user functions */
	{ "NICK",       (long) 0,       m_nick },
	{ "MODE",       (long) 0,       m_mode },
	{ "QUIT",       (long) 0,       m_quit },
#if defined(IRCD_HYB7) || defined(IRCD_OPN)
        { "AWAY",       (long) 0,       m_away },
#endif
        { "KILL",       (long) 0,       m_kill },
	/* Server traffic doesn't get used too much */
        { "SERVER",     (long) 0,       m_server },
        { "SQUIT",      (long) 0,       m_squit },
#ifdef TRACK_VERSION
        { "351",        (long) 0,       m_351 },
#endif
	{ "NOTICE",     (long) 0,       m_notice },
#ifdef IRCD_2_10
	{ "SERVICE",	(long) 0,	m_service },
#endif
	{ "VERSION",	(long) 0,	m_version },
	{ "MOTD",	(long) 0,	m_motd },
	{ "ADMIN",	(long) 0,	m_admin },
	{ "STATS",	(long) 0,	m_stats },
#ifdef SPLITSERV
	{ "INVITE",	(long) 0,	m_invite },
#endif
        { "WHOIS",	(long) 0,	m_whois },
	{ "TRACE",	(long) 0,	m_trace },
	{ "TIME",       (long) 0,       m_time },
	{ "INFO",	(long) 0,	m_info },
	/* PASS should only be received once (unless the uplink dies) */
	{ "PASS",	(long) 0,	m_pass },
        { (char *)0,    (long) 0,	(int (*)()) 0 }
};

/* The new parse should be much cleaner and more efficient....
** when it's done that is.......
*/
void parse(char *line)
{
	char *origin = NULL;
	char *pos;
        char *command = NULL;
        char *message = NULL;
        char *parv[MAXPARA];
	static char coreLine[BUFSIZE + 1];
        int parc = 0;
        int i;
	int status = -1;

	/* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* Clear out the error string each run through parse() */
	errstr[0] = '\0';

	if (line)
	{
                /* Sometimes we'll get a blank line with just a \n on it...
                ** catch those here....they'll core us later on if we don't
                */
                if (*line == '\n')
                        return;
                if (*line == '\000')
                        return;

	        /* For debugging purposes... */
		strncpy(coreLine, line, BUFSIZE);
		coreLine[BUFSIZE] = '\0';

		if (IsDebug())
		{
        		log("R: %s", line);
		}

		/* Find the first space */
		if ((pos = strchr(line, ' ')))
		{
			*pos = '\0';
                        pos++;
                        /* If it starts with a :...we have a prefix/origin
                        ** pull the origin off into origin, and have pos for
                        ** the command, message will be the part afterwards
                        */
                        if (*line == ':')
                        {
                                origin = line;
                                origin++;
                                if ((message = strchr(pos, ' ')))
				{
                                        *message = '\0';
                                        message++;
                                        command = pos;
                                }
				else
				{
					command = pos;
					message = NULL;
				}
                        }
			else
			{
                                        message = pos;
                                        command = line;
                        }
                }
                /* Ok, all the nasty part is over, now we need to make a
                ** parv out of what's left
                */

                if (message)
		{
                        if (*message == ':')
			{
                                message++;
                                parv[0] = message;
                                parc = 1;
                        }
			else
			{
                                parc = tokenize(message, parv);
                        }
		}
		else
		{
                        parc = 0;
                }

                /* Now we should have origin (or NULL), command, and a parv
                ** with it's accompanying parc...lets make ABSOLUTELY sure
                */
		if (!command)
		{
			log("Command not found: %s", coreLine);
			return;
		}

		/* Now we would take the command and compare it to what
		** we know (or preferred, loop through a table and call
		** the handlers in there
		*/
                for (i=0; msg_table[i].handler; i++)
                {
                        if (!strcasecmp(command, msg_table[i].name))
                        {
                            status = msg_table[i].handler(origin, parc, parv);
			    msg_table[i].counter++;
			    if (status == 1)
				log("Error with command %s: %s", command,
					errstr ? errstr : "No error message");
			    break;
                        }
                }
		
	}
	else
	{
		/* Something is REALLY weird here...we got a line that's
		** NULL...for now...just return;
		*/
	}
}

/* All handlers accept the origin as the first parameter (or NULL if there
** is no prefix.  Next is a parameter count, and a parameter vector
** Each handler returns 0 on success or 1 on failure
*/

int m_time(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		sts(":%s %d %s %s :%s", settings.name, RPL_TIME, origin, settings.name,
			irc_time(time(NULL)));
		return 0;
	}

	strcpy(errstr, "No Origin for TIME");
	return 1;
}

int m_version(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		sts(":%s %d %s %s(%s) :%s %s -> %s %s", settings.name, RPL_VERSION,
			origin, PATCHLEVEL, SERIALNUM, settings.name, VersionFlags(),
			version_date, version_time);
		return 0;
	}

	strcpy(errstr, "No Origin for VERSION");
	return 1;
}

int m_whois(char *origin, int parc, char *parv[])
{
	if (!parv[1])
	{
		strcpy(errstr, "No Target for WHOIS");
		return 1;
	}
	if (origin)
	{
		User *u=finduser(origin);

                if (!strcasecmp(parv[1], AdminServ.nick))
                {
                        /* Here is where to answer the WHOIS query */
                        sts(":%s %d %s %s %s %s * :%s", settings.name,
                                RPL_WHOISUSER, u->nick, AdminServ.nick,
                                AdminServ.user, AdminServ.host,
				"Administration Service");
                        sts(":%s %d %s %s %s :%s", settings.name,
                                RPL_WHOISSERVER, u->nick, AdminServ.nick,
                                settings.name, settings.desc);
			/* Always true for AdminServ */
                        sts(":%s %d %s %s :is an IRC Operator",
                                settings.name, RPL_WHOISOPERATOR,
                                u->nick, AdminServ.nick);
                        sts(":%s %d %s %s %d %ld :seconds idle, signon time",
                                settings.name, RPL_WHOISIDLE, u->nick,
                                AdminServ.nick, 0, me.t_start);
                        sts(":%s %d %s %s :End of WHOIS", settings.name,
                                RPL_ENDOFWHOIS, u->nick, AdminServ.nick);
                        return 0;
                }
#ifdef STATSERV
		if (!strcasecmp(parv[1], StatServ.nick))
		{
			/* Here is where to answer the WHOIS query */
			sts(":%s %d %s %s %s %s * :%s", settings.name,
				RPL_WHOISUSER, u->nick, StatServ.nick,
				StatServ.user, StatServ.host, StatServ.rname);
			sts(":%s %d %s %s %s :%s", settings.name,
				RPL_WHOISSERVER, u->nick, StatServ.nick,
				settings.name, settings.desc);
			if (IsSetOper())
			{
				sts(":%s %d %s %s :is an IRC Operator",
					settings.name, RPL_WHOISOPERATOR,
					u->nick, StatServ.nick);
			}
			sts(":%s %d %s %s %d %ld :seconds idle, signon time",
				settings.name, RPL_WHOISIDLE, u->nick,
				StatServ.nick, 0, me.t_start);
			sts(":%s %d %s %s :End of WHOIS", settings.name,
				RPL_ENDOFWHOIS, u->nick, StatServ.nick);
			return 0;
		}
#endif
#ifdef SPLITSERV
                if (!strcasecmp(parv[1], SplitServ.nick))
		{
                        /* Here is where to answer the WHOIS query */
                        sts(":%s %d %s %s %s %s * :%s", settings.name,
                                RPL_WHOISUSER, u->nick, SplitServ.nick,
                                SplitServ.user, SplitServ.host, sp.desc);
                        sts(":%s %d %s %s %s :%s", settings.name,
                                RPL_WHOISSERVER, u->nick, SplitServ.nick,
                                settings.name, settings.desc);
                        sts(":%s %d %s %s %d %ld :seconds idle, signon time",
                                settings.name, RPL_WHOISIDLE, u->nick,
                                SplitServ.nick, 0, me.t_start);
                        sts(":%s %d %s %s :End of WHOIS", settings.name,
                                RPL_ENDOFWHOIS, u->nick, SplitServ.nick);
                        return 0;
                }
#endif
		/* Don't answer any others (including the spam client) */
		return 0;
	}

	strcpy(errstr, "No Origin for WHOIS");
	return 1;
}

int m_info(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		const char **text = infotext;
		while (*text)
		{
	                if (**text)
        	                sts(":%s %d %s :%s", settings.name, RPL_INFO,
					origin, *text);
                	else
                        	sts(":%s %d %s : ", settings.name, RPL_INFO,
					origin);
                	text++;
		}
		sts("%s %d %s :End of /INFO list.", settings.name,
			RPL_ENDOFINFO, origin);
		return 0;
        }

        strcpy(errstr, "No Origin for INFO");
        return 1;
}

int m_motd(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		ShowMOTD(origin);
		return 0;
	}

	strcpy(errstr, "No Origin for MOTD");
	return 1;
}

int m_privmsg(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		/* parv[0] is the target of the PRIVMSG
		** parv[1] is the complete message
		*/

		/* If these don't exist, we have problems */
		if (!parv[0] || !parv[1])
		{
			strcpy(errstr, "Missing parameter in PRIVMSG");
			return 1;
		}

#ifdef STATSERV
#ifdef DISALLOW_NICK_PRIVMSG
		if (!strcasecmp(parv[0], StatServ.nick))
		{
			privmsg(origin, StatServ.nick, 
				"Please use /msg %s@%s message",
				StatServ.user, settings.name);
			privmsg(origin, StatServ.nick,
				"Message not delivered.");
			return 0;
		}
#else
		/* First check for PRIVATE messages to the Stats client */
		if (!strcasecmp(parv[0], StatServ.nick))
		{
			statserv(origin, parv[1]);
			return 0;
		}
#endif
#endif /* STATSERV */

		/* This checks for channel CTCP messages */
		if ( is_channel(parv[0]) )
		{
#ifdef STATSERV
			if (parv[1][0] == '\001')
			{
				statserv(origin, parv[1]);
			}
#endif
			/* Channel messages are ignored...but are NOT errors */
#ifdef SPLITSERV
			/* Unless they are in the Split service's channel */
	                if (!strcasecmp(parv[0], sp.join_chan))
			{
	                        splitserv(origin, parv[0], parv[1]);
      		                return 0;
                	}
#endif
			return 0;
		}

#ifdef SPLITSERV
                if (!strcasecmp(parv[0], SplitServ.nick))
		{
                        splitserv(origin, parv[0], parv[1]);
                        return 0;
                }
#endif

		if (!strcasecmp(parv[0], AdminServ.nick))
		{
			adminserv(origin, parv[1]);
			return 0;
		}

#ifdef SPAMSERV
		/* Now check for messages to the spam service */
		if (!strcasecmp(parv[0], s_Spam))
		{
			spam(origin, parv[1]);
			return 0;
		}
#endif

#ifdef SUPPORT_MSG_UH
#ifdef STATSERV
		if (!strcasecmp(parv[0], s_UH))
		{
			statserv(origin, parv[1]);
			return 0;
		}
#endif
#ifdef SPLITSERV
		if (!strcasecmp(parv[0], sp_UH))
		{
                        splitserv(origin, parv[0], parv[1]);
                        return 0;
                }
#endif
                if (!strcasecmp(parv[0], a_UH))
		{
                        adminserv(origin, parv[1]);
                        return 0;
                }
#endif
#ifdef SUPPORT_MSG_OPERS
		/* This will allow PRIVMSG opers@server in Hybrid 6 and (soon
		** to be fixed... Hybrid 7) to work.
		*/
		if (!strcasecmp(parv[0], s_OPERS))
		{
			MsgOpers(origin, parv[1]);
			return 0;
		}
#endif
	}

	/* We should NOT get here... The only way a PRIVMSG to
	** something on the service can get here is we used QUOTE
	** to create the user (or sent it to the spam's user@host)
	** We will ignore those messages
	*/

	snprintf(errstr, BUFSIZE, "Unknown PRIVMSG target: %s", parv[0]);
	return 1;
}

int m_notice(char *origin, int parc, char *parv[])
{
	if (!parv[0] || !parv[1])
	{
		strcpy(errstr, "Missing parameters in NOTICE");
		return 1;
	}
	/* Check for the NOTICE AUTH :... at the beginning of a connection.
	** It's normal and is NOT an error. NOTICE AUTH doesn't have an
	** origin so the check for an origin must be below this check
	*/
	if (!strcasecmp(parv[0], "AUTH"))
		return 0;
	if (!origin)
	{
		strcpy(errstr, "NOTICE without an origin");
		return 1;
	}
#ifdef SUPPORT_MSG_OPERS
	if (!strcasecmp(parv[0], s_OPERS))
	{
		MsgOpers(origin, parv[1]);
		return 0;
	}
#endif
	snprintf(errstr, BUFSIZE, "Unknown NOTICE: %s - %s - %s", origin, 
		parv[0], parv[1]);
	return 1;
}

int m_server(char *origin, int parc, char *parv[])
{
	/* Hybrid based servers:  parv[0] = server name, parv[1] = hops,
	** parv[2] = description, with or without an origin
	** IRCnet with an origin: parv[0] = server name, parv[1] = hops,
	** parv[2] = server token, parv[3] = description, Without an origin
	** IRCnet acts like the rest.
	*/
	if (!parv[0] || !parv[1] || !parv[2])
	{
		strcpy(errstr, "Missing parameters in SERVER");
		return 1;
	}
	if (origin)
	{
#ifdef SPLITSERV
		time_t now = time(NULL);
		SpServer *spl = NULL;
		char *td = NULL;
#endif
#ifdef IRCD_2_10
		if (!parv[3])
		{
			strcpy(errstr, "Missing parameter in SERVER");
			return 1;
		}
		AddServer(parv[0], atoi(parv[1]), atoi(parv[2]), parv[3],
			origin);
#else
		AddServer(parv[0], atoi(parv[1]), parv[2], origin);
#endif
#ifdef SPLITSERV
		spl = SpFindSplit(parv[0]);
		if (spl)
		{
			td = timediff(now - spl->ts);
			splog("End of Split: %s to %s: %s", parv[0],
				origin, td);
			if (IsSPReport())
			    spprivmsg(sp.join_chan, SplitServ.nick,
                                        "END OF SPLIT: %s to %s: %s", parv[0],
					origin, td);
		}
		else
		{
			if (!is_delayed)
			{
				splog("New Server: %s to %s", parv[0], origin);
				if (IsSPReport())
				    spprivmsg(sp.join_chan, SplitServ.nick,
					"CONNECT: %s to %s", parv[0], origin);
			}
		}
		SpNewServer(parv[0], origin);
		SpDelSplit(parv[0]);
#endif
		return 0;
	}
	/* Now parse SERVER without an origin (our uplink) */
#ifdef IRCD_2_10
		/* The uplink is always token 1 */
	AddServer(parv[0], atoi(parv[1]), 1, parv[2], settings.name);
#else
	AddServer(parv[0], atoi(parv[1]), parv[2], settings.name);
#endif
	/* This is our uplink, so set me.s */
	me.s = findserver(parv[0]);
#ifdef SPLITSERV
	splog("New Server: %s", parv[0]);
	if (!is_delayed)
	{
		if (IsSPReport())
		    spprivmsg(sp.join_chan, SplitServ.nick,
			"CONNECT: %s to %s", parv[0], settings.name);
	}
	SpNewServer(parv[0], settings.name);
	SpDelSplit(parv[0]);
	map_uplink=SpFindServer(parv[0]);
#endif
	return 0;
}

int m_squit(char *origin, int parc, char *parv[])
{
	Server *s = NULL;
	/* This one works the same if we have an origin or not */

#ifdef SPLITSERV
#if defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_2_10)
	SpServer* sv = NULL;
	static char b2[BUFSIZE + 1];
	int i;
#endif
#endif

	if ((s = findserver(parv[0]))) s->stats->numsplits++;
	DelServer(parv[0]);

#ifdef SPLITSERV
#if defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_2_10)
	sv = SpFindServer(parv[0]);
	if (sv)
	{
		for (i=0; i<MAXSERVER; i++)
		{
			if (sv->leaflist[i])
			{
				sprintf(b2, "SQUIT %s :QS",
					sv->leaflist[i]->name);
				parse(b2);
			}
		}
	}
#endif

#if defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_2_10)
                        splog("Split: %s", parv[0]);
                        if (!is_delayed)
			{
				if (IsSPReport())
                                    spprivmsg(sp.join_chan, SplitServ.nick,
					"SPLIT: %s", parv[0]);
                        }
#else /* IRCD_HYB6 */
                        splog("Split: %s - %s", parv[0], parv[1]);
                        if (!is_delayed)
			{
				if (IsSPReport())
                                    spprivmsg(sp.join_chan, SplitServ.nick,
					"SPLIT: %s - %s", parv[0], parv[1]);
                        }
#endif /* IRCD_HYB6 */

	SpDelServer(parv[0]);
	SpNewSplit(parv[0]);
#endif

#ifdef IRCD_2_10
	DelServicesByServer(parv[0]);
#endif

	return 0;
}

int m_quit(char *origin, int parc, char *parv[])
{
	if (origin)
	{
#ifdef IRCD_2_10
		char *host = NULL;
		if ((host = strchr(origin, '@')))
		{
			*host = '\0';
			host++;
			DelService(origin, host);
			return 0;
		}
#endif
		DelUser(origin);
		return 0;
	}

	strcpy(errstr, "QUIT without an origin");
	return 1;
}

#ifdef TRACK_VERSION
int m_351(char *origin, int parc, char *parv[])
{
	/* parv[0] = the target (who sent the VERSION query)
	** parv[1], [2], and [3] = The three parts of the VERSION reply
	*/
	Server *s = NULL;
	if (origin)
	{
		s = findserver(origin);
		if (s)
		{
			snprintf(s->version, BUFSIZE, "%s %s %s", parv[1],
				parv[2], parv[3]);
			s->version[BUFSIZE] = '\0';
			return 0;
		}
	}

	strcpy(errstr, "351 Numeric without an origin");
	return 1;
}
#endif

int m_mode(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		if (!parv[0] || !parv[1])
		{
			strcpy(errstr, "Missing parameters in MODE");
			return 1;
		}
		if ( !is_channel(parv[0]) )
			UserMode(origin, parv[1]);
		return 0;
	}

	strcpy(errstr, "MODE without an origin");
	return 1;
}

#ifdef SPLITSERV
int m_invite(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		if (!strcasecmp(parv[0], SplitServ.nick) &&
			!strcasecmp(parv[1], sp.join_chan))
		{
			init_spjoin();
		}

		return 0;
	}

	strcpy(errstr, "INVITE without an origin");
	return 1;
}
#endif

int m_kill(char *origin, int parc, char *parv[])
{
	User *u = NULL;
	Server *s = NULL;

	if (!parv[0])
	{
		strcpy(errstr, "No target in KILL");
		return 1;
	}
	if (origin)
	{
		u = finduser(origin);
		if (u)
		{
			u->server->stats->operkills++;
		}
		else
		{
			s = findserver(origin);
			if (s)
				s->stats->serverkills++;
		}
		DelUser(parv[0]);
		if (!strcasecmp(parv[0], AdminServ.nick))
		{
                        init_admin();
#ifdef DIE_ON_MASS_COLLIDE
                        netinfo.collides++;
                        if (netinfo.collides > MAXCOLLISIONS)
			{
                                CollideDie();
                        }
#endif
#ifdef STATSERV
		}
		else if (!strcasecmp(parv[0], StatServ.nick))
		{
			init_statserv();
			init_join();
#ifdef DIE_ON_MASS_COLLIDE
			netinfo.collides++;
			if (netinfo.collides > MAXCOLLISIONS)
			{
				CollideDie();
			}
#endif
#endif
#ifdef SPAMSERV
		}
		else if (!strcasecmp(parv[0], s_Spam))
		{
			init_spam();
#ifdef DIE_ON_MASS_COLLIDE
			netinfo.collides++;
			if (netinfo.collides > MAXCOLLISIONS)
			{
                                CollideDie();
                        }
#endif
#endif
#ifdef SPLITSERV
		}
		else if (!strcasecmp(parv[0], SplitServ.nick))
		{
			init_splitserv();
			init_spjoin();
#ifdef DIE_ON_MASS_COLLIDE
                        netinfo.collides++;
                        if (netinfo.collides > MAXCOLLISIONS)
			{
                                CollideDie();
                        }
#endif
#endif
                }
		else if (!strcasecmp(parv[0], AdminServ.nick))
		{
                        init_admin();
#ifdef DIE_ON_MASS_COLLIDE
                        netinfo.collides++;
                        if (netinfo.collides > MAXCOLLISIONS)
			{
                                CollideDie();
                        }
#endif
		}
		return 0;
	}

	strcpy(errstr, "KILL without an origin");
	return 1;
}

#ifdef LAG_CHECK
int m_pong(char *origin, int parc, char *parv[])
{
	/* Works with or without an origin, as long as the lag detecting
	** PING's are sent right
	*/
	Server *s = NULL;
	if (!parv[0])
	{
		strcpy(errstr, "PONG without a parameter");
		return 1;
	}
	s = findserver(parv[0]);
	if (s)
	{
		s->ping = time(NULL) - ping.last_sent;
		if (ping.ulag > 1)
			s->ping -= (float) ping.ulag;
		if (!strcmp(me.s->name, s->name))
			ping.ulag = me.s->ping;
		if (s->ping > s->highest_ping)
		{
			s->highest_ping = s->ping;
			s->t_highest_ping = time(NULL);
		}
		else if (s->ping < s->lowest_ping)
		{
			s->lowest_ping = s->ping;
			s->t_lowest_ping = time(NULL);
		}
		return 0;
	}
	else
	{
		log("Received PONG from unknown server: %s", parv[0]);
		snprintf(errstr, BUFSIZE,
			"Received PONG from unknown server: %s", parv[0]);
		return 1;
	}
}
#endif

#if defined(IRCD_HYB7) || defined(IRCD_OPN)
int m_away(char *origin, int parc, char *parv[])
{
	/* Hybrid 5 and 6 do not propagate AWAY.  IRCnet does, but via the
	** +a usermode
	*/
	User *u = NULL;
	if (origin)
	{
		u = finduser(origin);
		if (u)
		{
			/* This might need to be done better */
			if (IsUIsAway())
			{
				netinfo.away--;
				u->flags &= ~UFLAG_ISAWAY;
			}
			else
			{
				netinfo.away++;
				u->flags |= UFLAG_ISAWAY;
			}
			return 0;
		}
	}

	strcpy(errstr, "AWAY without an origin");
	return 1;
}
#endif

int m_nick(char *origin, int parc, char *parv[])
{
	User *u = NULL;
#ifdef IRCD_2_10
	Server *s = NULL;
#endif
	if (origin)
	{
		if (!parv[0])
			return 1;
		/* With an origin, it's a nick change 
		** NOTE:  IRCnet puts a : in front of the new nick, while
		** The others don't.  It doesn't matter with the new parser
		** as it will properly deal with it.  The old one didn't
		*/
		if ( (u = finduser(origin)) )
		{
			Change_User(u, parv[0]);
			return 0;
		}
		snprintf(errstr, BUFSIZE, "Unknown user: %s", parv[0]);
		return 1;
	}
	/* Without an origin, it's a nick introduction...
	** parv contents vary for each server type...
	** Hybrid:  0 = nick, 1 = hops, 2 = TS, 3 = modes, 4 = user
	**          5 = host, 6 = server, 7 = gecos
	** IRCnet: 0 = nick, 1 = hops, 2 = user, 3 = host, 4 = server token,
	**         5 = modes, 6 = gecos
	*/

	if (!parv[0] || !parv[1] || !parv[2] || !parv[3] || !parv[4]
                     || !parv[5] || !parv[6])
	{
		strcpy(errstr, "Missing parameter in NICK introduction");
		return 1;
	}
#ifndef IRCD_2_10
	if (!parv[7])
	{
		strcpy(errstr, "Missing parameter in NICK introduction");
		return 1;
	}
#endif

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	AddUser(parv[0], parv[4], parv[5], parv[6]);
	UserMode(parv[0], parv[3]);
#else
#if defined(IRCD_2_10)
	s = findservtoken(atoi(parv[4]));
	AddUser(parv[0], parv[2], parv[3], s);
	UserMode(parv[0], parv[5]);
#endif
#endif
	return 0;
}

#ifdef IRCD_2_10
int m_service(char *origin, int parc, char *parv[])
{
	Service *sv = NULL;
	char *host = NULL;

	host = strchr(parv[0], '@');
	if (host)
	{
		*host = '\0';
		host ++;
	}
	else
	{
		/* Something weird is going on... */
		strcpy(errstr, "SERVICE received without @");
		return 1;
	}

	sv = findservice(parv[0], host);

	if (sv)
	{
		strcpy(errstr, "Reintroduced Service!");
		return 1;
	}

	/* Parameters:  parv[0] (after processing) = nick, host = server,
	**              parv[1] = hopcount, parv[2] = distribution,
	**		parv[3] = parameters, parv[4] = server token,
	**		parv[5] = gecos
	*/

	AddService(parv[0], host, parv[2], parv[5]);

	return 0;
}
#endif

int m_ping(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		/* It's a remote Lag detecting PING */
		if (!parv[0] || !parv[1])
		{
			strcpy(errstr, "Missing parameter in PING");
			return 1;
		}
		/* Lag detecting PING's we reverse the order of the two
		** parameters
		*/
		sts(":%s PONG %s :%s", settings.name, parv[1], parv[0]);
		return 0;
	}
	/* Without an origin, it's our uplink checking us...be sure to
	** answer or we'll ping timeout...this PING is like a heartbeat 
	** to the service as well.
	*/

	if (!parv[0]) {
		strcpy(errstr, "Missing parameter in PING");
		return 1;
	}
	sts("PONG %s", parv[0]);
	return 0;
}

int m_admin(char *origin, int parc, char *parv[])
{
	if (origin)
	{
		/* Is it directed to us or a jupe behind us? */
		if (parv[0])
		{
			/* We should NEVER get an ADMIN query that doesn't
			** have a valid parameter
			*/
			if (!strcasecmp(settings.name, parv[0]))
			{
			    sts(":%s %d %s :Administrative info about %s",
				settings.name, RPL_ADMINME, origin,
				settings.name);
			    sts(":%s %d %s :%s Network Statistical Services",
				settings.name, RPL_ADMINLOC1, origin,
				settings.netname);
			    sts(":%s %d %s :%s", settings.name, RPL_ADMINLOC2,
				origin, settings.admin_name);
			    sts(":%s %d %s :%s", settings.name, RPL_ADMINEMAIL,
				origin, settings.admin_email);
			    return 0;
			}
			/* Query'ing our JUPE's isn't really an error */
#ifdef ENABLE_JUPE
			return 0;
#else
			snprintf(errstr, BUFSIZE,
				"Received ADMIN request for unknown leaf: %s",
				parv[0]);
			return 1;
#endif
		}
		strcpy(errstr, "Missing parameter in ADMIN");
		return 1;
	}
	strcpy(errstr, "ADMIN without an origin");
	return 1;
}

int m_stats(char *origin, int parc, char *parv[])
{
	/* parv[0] = type, parv[1] = target */
	User *u = NULL;
	if (!parv[0] || !parv[1])
	{
		strcpy(errstr, "Missing parameter in STATS");
		return 1;
	}
	if (origin)
	{
		u = finduser(origin);
		if (u)
		{
			if (!strcasecmp(settings.name, parv[1]))
			{
				ParseStats(u, parv[0][0]);
				return 0;
			}
		}
		snprintf(errstr, BUFSIZE, "Unknown user: %s in STATS", origin);
		return 1;
	}
	strcpy(errstr, "No origin in STATS");
	return 1;
}

int m_trace(char *origin, int parc, char *parv[])
{
	User *u = NULL;
	if (!parv[0])
	{
		strcpy(errstr, "Missing parameter in TRACE");
		return 1;
	}
	if (origin)
	{
		u = finduser(origin);
		if (u)
		{
			if (!strcasecmp(settings.name, parv[0]))
			{
				Trace(u);
				return 0;
			}
		}
		snprintf(errstr, BUFSIZE, "Unknown user: %s in TRACE", origin);
		return 1;
	}
	strcpy(errstr, "No origin in TRACE");
	return 1;
}

int m_pass(char *origin, int parc, char *parv[])
{
	/* When we receive the PASS command without an origin (should NEVER
	** be received with an origin), then we can initialize our burst
	*/
	if (!origin)
	{
#ifdef STATSERV
		init_statserv();
#endif
#ifdef SPAMSERV
		init_spam();
#endif
#ifdef SPLITSERV
		init_splitserv();
		init_spjoin();
#endif
		init_admin();
#ifdef STATSERV
		init_join();
#endif
		return 0;
	}
	strcpy(errstr, "Origin specified in PASS");
	return 1;
}

/* Send a message to a user from a user */
void privmsg(char *to, const char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];
#ifdef DELAY_PRIVMSG
	struct timeval tv;
#endif

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
	if (IsPrivmsg())
		snprintf(buf, BUFSIZE, ":%s PRIVMSG %s :%s", from, to, buf2);
	else
		snprintf(buf, BUFSIZE, ":%s NOTICE %s :%s", from, to, buf2);
	sts("%s", buf);
#ifdef DELAY_PRIVMSG
	tv.tv_usec = DELAYTIME; /* 3/4 a second delay */
	tv.tv_sec = 0;       /* ignore this */
	select(1, NULL, NULL, NULL, &tv);
#endif
	va_end(ap);
}

/* This sends an array of character arrays in a PRIVMSG or NOTICE
** to a user from a user created by the service (typically StatServ.nick)
** but it's generic enough to be used for future extensions
*/
void privmsg_list(char *to, char *from, const char **text)
{
	while (*text)
	{
		if (**text)
			privmsg(to, from, "%s", *text);
		else
			privmsg(to, from, " ");
		text++;
	}	
}

void wallops(char *from, char *fmt, ...)
{
	va_list ap;
	char buf[512], buf2[512];

	va_start(ap, fmt);
	vsnprintf(buf2, sizeof(buf2), fmt, ap);
	snprintf(buf, BUFSIZE, ":%s WALLOPS :%s", from, buf2);
	sts("%s", buf);
	va_end(ap);
}

#ifdef STATSERV
/* This function KILL's or optionally ignores a user for flooding services. */
/* This needs to be adjusted for SplitServ too! */
int flood(User *u)
{
	/* Our "opers" are immune to flood control */
	if (u->myuser) 
		return 0;

	/* Flood control is disabled by setting the flood lines to 0 */
	if (settings.flood)
	{
		time_t current = time(NULL);

		if (current - u->t_flood > settings.flood_time)
		{
			u->t_flood = time(NULL);
			u->flood = 0;
			u->flags &= ~UFLAG_FLOODSENT;
			return 0;
		}

		if (u->flood >= settings.flood)
		{
		   /* This was a switch in the initial writing of this
		   ** For now (unless I see a specific reason against
		   ** using switch, it can stay a switch)
		   */
		   if (!IsFlood())
		   {
			sts(":%s KILL %s :%s!%s (Flooding Services.)",
				StatServ.nick, u->nick, StatServ.host, 
				StatServ.nick);
			log("FLOODING (KILLED): %s!%s@%s", u->nick, 
				u->username, u->hostname);
			/* This is an unnecessary piece of overhead ugh */
			/* We KILL the user, we should remove them from 
			** the databases...
			*/
			DelUser(u->nick);
		   }
		   else
		   {
			/* Only send the flood WALLOPS/GLOBOPS message
			** once, or WE will be the ones flooding
			*/
			if (!(u->flags & UFLAG_FLOODSENT)) {
			  wallops("%s is Flooding Services", u->nick);
			  log("FLOODING (IGNORED): %s!%s@%s", u->nick,
				  u->username, u->hostname);
			}
		   }
		   return 1;
		}
		else
		{
			u->flood++;
		}
	}
	return 0;
}
#endif

/* This is left in unmodified as to the original author's request */
void ShowMOTD(char *nick)
{
	static unsigned char copyright[38] = {  /* Can you tell I was bored? */
	0x47, 0x65, 0x6f, 0x53, 0x74, 0x61, 0x74, 0x73, 0x20, 0x61, 0x72, 0x65,
	0x20, 0x28, 0x63, 0x29, 0x20, 0x31, 0x39, 0x39, 0x39, 0x20, 0x4a, 0x6f,
	0x6e, 0x61, 0x74, 0x68, 0x61, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
	0x65, 0x0
	};

	/* Removal or modification of this function is against the law.
	** Be nice, and don't remove it or modify it.
	** -Jonathan 'netgod' George (net@lite.net) */

	sts(":%s 375 %s :- %s Message of the Day", settings.name, nick, settings.name);
	sts(":%s 372 %s :- ", settings.name, nick);
	sts(":%s 372 %s :- %s", settings.name, nick, copyright);
	sts(":%s 372 %s :- Grab your copy at ftp.lite.net", settings.name, nick);
	sts(":%s 376 %s :End of /MOTD command.", settings.name, nick);
}

/* Send out QUIT messages as a courtesy... The only one who really 
** matters is StatServ.nick, as s_Spam will never be in a channel...
** unless the Spam service is rewritten to join channels looking for 
** ON JOIN spam... Then we get into some heavy privacy issues :/
*/
void SendQuit(User *u, int restart, char * comm)
{
#ifdef STATSERV
	sts(":%s QUIT :%s Request by %s: %s", StatServ.nick, 
	    (restart) ? "Restart" : "Shutdown", u->nick,
            (comm && comm[0]) ? comm : "No Reason");
#endif
#ifdef SPAMSERV
	sts(":%s QUIT :%s Request by %s: %s", s_Spam, 
	       (restart) ? "Restart" : "Shutdown", u->nick,
               (comm && comm[0]) ? comm : "No Reason");
#endif
#ifdef SPLITSERV
	sts(":%s QUIT :%s Request by %s: %s", SplitServ.nick,
	    (restart) ? "Restart" : "Shutdown", u->nick,
	    (comm && comm[0]) ? comm : "No Reason");
#endif
	sts(":%s QUIT :%s Request by %s: %s", AdminServ.nick,
            (restart) ? "Restart" : "Shutdown", u->nick,
            (comm && comm[0]) ? comm : "No Reason");
}

/* This is how we answer STATS requests...
** This is very Hybrid-like at the moment.  It should not affect other
** ircd's.  If someone can refine this for other ircd's, I will patch it in
*/
void ParseStats(User *u, char type)
{
    MyUser *myu = NULL;
    int count = 0;
    int i = 0;

    switch(type)
    {
	case 'c':
		/* Our only "C and N" lines are to our uplink, show the
		** real IP/address of our uplink to opers, but use the
		** Hybrid6 method of IP hiding for non-opers.  This is
		** moot if your ircd blocks remote STATS from non-opers
		*/
		sts(":%s %d %s C %s * %s %d 2", settings.name, RPL_STATSCLINE, 
			u->nick, 
			IsUIsOper() ? settings.uplink : "*@127.0.0.1", 
			me.s->name, settings.port);
		sts(":%s %d %s N %s * %s 0 2", settings.name, RPL_STATSNLINE, 
			u->nick, 
			IsUIsOper() ? settings.uplink : "*@127.0.0.1", 
			me.s->name);
		sts(":%s %d %s c :End of STATS report", settings.name, 
			RPL_ENDOFSTATS, u->nick);
		break;
	case 'i':
		/* This is purely cosmetic, show a closed server */
		sts(":%s %d %s I * * *@%s 0 1", settings.name, RPL_STATSILINE, 
			u->nick, settings.name);
                sts(":%s %d %s i :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	case 'h':
		/* Our only hub should be our uplink.  This is cosmetic
		** as well.
		*/
		sts(":%s %d %s H * * %s 0 -1", settings.name, RPL_STATSHLINE,
			u->nick, me.s->name);
                sts(":%s %d %s h :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	case 'm':
		/* Show the known commands and their usage counters */
		for (i=0; msg_table[i].handler; i++)
		{
		    /* lie about the byte count */
		    sts(":%s %d %s %s %ld 0", settings.name, RPL_STATSCOMMANDS,
			u->nick, msg_table[i].name, msg_table[i].counter);
		}
		sts(":%s %d %s y :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	case 'o':
	    /* This STATS is actually very useful.  This shows a list of
	    ** our users (all who can use LOGIN, the equivalent of opers)
	    */
	    for (myu = myuhead; myu; myu = myu->next)
	    {
		sts(":%s %d %s O *@* * %s 0 10", settings.name, RPL_STATSOLINE,
			u->nick, myu->username);
	    }
            sts(":%s %d %s o :End of STATS report", settings.name,
                    RPL_ENDOFSTATS, u->nick);
            break;
	case 'p':
	    /* This STATS is also relatively useful.  It does the Hybrid
	    ** STATS p - show all online opers.  All users who have used
	    ** LOGIN will be shown as Stats opers.
	    ** NOTE:  We now DO print out the current user's nick.
	    */
	    for (myu = myuhead; myu; myu = myu->next)
	    {
		if (IsMIsOn())
		{
		    sts(":%s %d %s :[O] %s", settings.name, RPL_STATSDEBUG,
			u->nick, myu->u->nick);
		    count++;
		}
	    }
	    sts(":%s %d %s :%d OPER%s", settings.name, RPL_STATSDEBUG, u->nick,
		count, (count == 1) ? "" : "s");
	    sts(":%s %d %s p :End of STATS report", settings.name,
		RPL_ENDOFSTATS, u->nick);
	    break;
	case 'u':
		/* Send our uptime in the usual fasion */
		sts(":%s %d %s :Statistics up %s", settings.name,
			RPL_STATSUPTIME, u->nick,
			timediff(time(NULL) - me.t_start));
		sts(":%s %d %s u :End of STATS report", settings.name,
			RPL_ENDOFSTATS, u->nick);
		break;
	case 't':
		/* For now, just show the byte counters */
		sts(":%s %d %s :Bytes in: %ld, Bytes out: %ld", settings.name,
			RPL_STATSDEBUG, u->nick, netinfo.inb, netinfo.outb);
		sts(":%s %d %s u :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
                break;
	case 'y':
		/* Purely cosmetic with fake values...EXCEPT for the 
		** reconnect time in the server Y line (2)
		*/
		sts(":%s %d %s Y 1 0 0 2 000000000", settings.name, RPL_STATSYLINE,
			u->nick);
		sts(":%s %d %s Y 2 0 %d 1 000000000", settings.name, 
			RPL_STATSYLINE, u->nick, settings.r_time);
		sts(":%s %d %s Y 10 0 0 1 000000000", settings.name, 
			RPL_STATSYLINE, u->nick);
		sts(":%s %d %s y :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
		break;
	default:
		sts(":%s %d %s * :End of STATS report", settings.name,
                        RPL_ENDOFSTATS, u->nick);
                break;
    }
}

/* Report TRACE in the Hybrid style */
void Trace(User *u)
{
    if (IsUIsOper())
    {
#ifdef STATSERV
	   sts(":%s %d %s %s %d %s[%s@%s] (127.0.0.1) 000000000 000000000",
		settings.name, 
		IsSetOper() ? RPL_TRACEOPERATOR : RPL_TRACEUSER, 
		u->nick, 
                IsSetOper() ? "Oper" : "User", 
		IsSetOper() ? 10 : 1, StatServ.nick, StatServ.user, 
		StatServ.host);
#endif
#ifdef SPAMSERV
	   sts(":%s %d %s User 1 %s[please@%s] (127.0.0.1) 000000000 000000000",
		    settings.name, RPL_TRACEUSER, u->nick, s_Spam, settings.name);
#endif
           sts(":%s %d %s Serv 2 %dS %ldC %s AutoConn.!*@%s 000000000",
                    settings.name, RPL_TRACESERVER, u->nick, netinfo.maxservers, 
		    netinfo.maxusers, me.s->name, settings.name);
	   sts(":%s %d %s %s :End of TRACE", settings.name, RPL_ENDOFTRACE, u->nick, 
		    settings.name);
    }
}

#ifdef SUPPORT_MSG_OPERS
void MsgOpers(char *nick, char *message)
{
	MyUser* myu = NULL;

	for (myu = myuhead; myu; myu = myu->next)
	{
		if (IsMIsOn())
			sts(":%s NOTICE %s :To opers: From %s: %s", settings.name,
                        	myu->u->nick, nick, message);
        }
}
#endif

void CollideDie()
{
                wallops(settings.name, "Collision Flood!  Gracefully Exiting");
                log("Collision Flood!  Gracefully Exiting");
#ifdef STATSERV
                SendQuit(finduser(StatServ.nick), 0, "Collision Flood");
#endif
#ifdef SPLITSERV
		SendQuit(finduser(SplitServ.nick), 0, "Collision Flood");
#endif
#ifdef SPAMSERV
		SendQuit(finduser(s_Spam), 0, "Collision Flood");
#endif
		SendQuit(finduser(AdminServ.nick), 0, "Collision Flood");
                sleep(1);
                close(servsock);
                SaveStats();
                exit(0);
}
