/* Stats - IRC Statistical Services
** Original Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** $Id: main.c,v 1.47 2001/09/14 15:08:27 wcampbel Exp $
**
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "patchlevel.h"
#include <signal.h>

#ifdef CYGWIN
#include <getopt.h>
#endif

#ifdef SPAMSERV
char s_Spam[8] = "cm-i";
#endif
int servsock;
const char version_date[] = __DATE__;
const char version_time[] = __TIME__;

#ifdef SUPPORT_MSG_UH
#ifdef STATSERV
char s_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef SPLITSERV
char sp_UH[MAXUSER + MAXHOST + 2];
#endif
char a_UH[MAXUSER + MAXHOST + 2];
#endif

#ifdef SUPPORT_MSG_OPERS
/* 7 is "opers@" and the \0 terminator
** I should PROBABLY force this to only be enabled on Hybrid 6 
*/
char s_OPERS[MAXHOST + 7];
#endif

/* NOTE:  This code needs profiled to find out what needs to be optimized
** to make the code as efficient as possible.
*/
static void start();
static void CmdLine(int, char **);
static void PrintVersion();
static void PrintHelp();

int main(int argc, char *argv[])
{
	int forked = 0;
	FILE *fp;
	
	me.cmdflags = 0x00000000;

	CmdLine(argc, argv);

	log("Statistics Started (%s(%s) %s).", PATCHLEVEL, SERIALNUM,
		VersionFlags());
	me.t_start = time(NULL);

	/* chdir returns 0 on sucess, -1 on failure */
	if (chdir(DPATH))
	{
		printf("Unable to chdir to DPATH, did you edit config.h?\n");
		printf("Fatal Error, exiting\n");
		exit(1);
	}

	InitDefaults();
	ConfLoad();
	netinfo.chan_count = GetChanList();

	/* Sanity checking for FLOOD */
	if (settings.flood == 1)
	{
		settings.flood = 2;
	}

#ifdef SUPPORT_MSG_UH
	/* Create the user@host that we are to save some cycles later...
	** IF this features is used often
	*/
#ifdef STATSERV
	snprintf(s_UH, (MAXHOST + MAXUSER + 2), "%s@%s", StatServ.user,
		settings.name);
#endif
#ifdef SPLITSERV
        snprintf(sp_UH, (MAXHOST + MAXUSER + 2), "%s@%s", SplitServ.user,
                settings.name);
#endif
        snprintf(a_UH, (MAXHOST + MAXUSER + 2), "%s@%s", AdminServ.user,
                settings.name);
#endif

#ifdef SUPPORT_MSG_OPERS
	/* Same as above...If we call it once, we'll definately call it 
	** again...
	*/
	snprintf(s_OPERS, (MAXHOST + 7), "opers@%s", settings.name);
#endif

	/* This should ONLY be run once */
	srandom(time(NULL)); 

	/* Zero out the byte counters */
	netinfo.inb = 0;
	netinfo.outb = 0;
#ifdef DIE_ON_MASS_COLLIDE
	netinfo.collides = 0;
#endif
#ifdef IRCD_2_10
	servicehead = NULL;
#endif

	LoadStats();
	LoadTLD();
	LoadMyUsers();

	/* Setup signals */
	signal(SIGTERM, SigTerm);

	if (!IsDebug() && !IsNoFork())
		forked=fork();
	if (forked)
	{
		fp = fopen(PIDFILE, "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	}

#if 0
	/* hmm, why doesn't this work */
	fclose(stdout);
	fclose(stdin);
	fclose(stderr);
#endif

#ifdef SPLITSERV
	SpInitTables();
	notice = time(NULL) + sp.delay;
	is_delayed = 1;
#endif

	start();

	return 1;
}

void start()
{
	TimerReset();
	netinfo.users = netinfo.opers = netinfo.servers = 0;
	init_server_hash();
	init_user_hash();
	init_tld();

	log("Connecting to %s:%d", settings.uplink, settings.port);
	if (servsock > 0)
		close(servsock);

	servsock = ConnectTo(settings.uplink, settings.port);

	if (servsock <= 0)
	{
		log("Unable to connect to %s port %d", settings.uplink, settings.port);
	}
	else
	{
		login();
		read_loop();
	}
	log("Reconnecting to the server in %d seconds", settings.r_time);
	/* Here I need to flush the server/user table */
	if (IsDebug())
	{
		if (!me.s) log("Invalid me.s");
	}
	if (me.s) FlushServer(me.s->name);

#ifdef SPLITSERV
        /* Now we get rid of all our uplink's SERVER table */
        SpFlushServerTree(map_uplink);
#endif
	sleep(settings.r_time);
#ifdef SPLITSERV
        /* Reset the variables used in initialization to prevent the
        ** service from spewing large chunks of data out to the channel
        */
        notice = time(NULL) + sp.delay + settings.r_time;
        is_delayed = 1;
#endif
	start();
}

/* This is ugly, logging into each server type is slightly different */
void login()
{
#if defined(IRCD_HYBRID)
	sts("PASS %s :TS", settings.pass);
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_HYB6)
	sts("PASS %s :TS", settings.pass);
	sts("CAPAB :QS");
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
	sts("SVINFO 3 1 0 :%ld", time(NULL));
#else
#if defined(IRCD_OPN)
	sts("PASS %s :TS", settings.pass);
        sts("CAPAB :QS OPN");
        sts("SERVER %s 1 :%s", settings.name, settings.desc);
        sts("SVINFO 4 4 0 :%ld", time(NULL));
#else
#if defined(IRCD_HYB7)
	/* NOTE:  a Stats service CANNOT be a LazyLink!!!!!!!!!
	** It's impossible for a Stats service to keep accurate
	** counts on anything if if doesn't receive a complete burst
	*/
	sts("PASS %s :TS", settings.pass);
	/* Make sure to specify it can be a hub - This will only show up in
	** the STATS ? CAPAB list for the Stats service if it is H: lined
	** Always specifying HUB won't hurt anything though
        */
	sts("CAPAB :QS HUB");
        sts("SERVER %s 1 :%s", settings.name, settings.desc);
        sts("SVINFO 5 3 0 :%ld", time(NULL));
#else
#if defined(IRCD_2_10)
	sts("PASS %s :021030002 IRC|%sHst P", settings.pass,
		IsDebug() ? "D" : "");
	sts("SERVER %s 1 :%s", settings.name, settings.desc);
#endif
#endif
#endif
#endif
#endif
}

#ifdef STATSERV
/* Init the StatServ client */
void init_statserv()
{

#ifdef SUPPORT_MSG_UH
	snprintf(StatServ.rname, LINE, "/msg %s HELP", s_UH);
#else
	snprintf(StatServ.rname, LINE, "/msg %s HELP", StatServ.nick);
#endif

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	sts("NICK %s 1 1 %s %s %s %s :%s", StatServ.nick,
		IsSetOper() ? "+o" : "+", StatServ.user, StatServ.host,
		settings.name, StatServ.rname);
#else
#if defined(IRCD_2_10)
	sts("NICK %s 1 %s %s %d %s :%s", StatServ.nick,
		StatServ.user, StatServ.host, 
		/* This is the server token (which is ugly as sin) 
		** For now, I'll send '1'... Once I find out more about
		** it, I'll send the proper value.  
		*/
		1, IsSetOper() ? "+o" : "+", StatServ.rname);
#endif
#endif

#ifdef IRCD_2_10
	AddUser(StatServ.nick, StatServ.user, StatServ.host,
		findserver(settings.name));
#else
	AddUser(StatServ.nick, StatServ.user, StatServ.host, settings.name);
#endif

/* If the StatServ client is an oper, increase oper count here
** and set it as an oper
*/
	if (IsSetOper()) {
		User* u = finduser(StatServ.nick);
		IncreaseOpers(findserver(settings.name));
		u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
		u->operflags |= OPER_GLOBAL;
#endif
	}
}
#endif /* STATSERV */

#ifdef SPAMSERV
/* Bring up the Spam detecting client if it was enabled */
void init_spam()
{

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	sts("NICK %s 1 1 + please %s %s :Spam Me!", s_Spam,
		settings.name, settings.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 please %s %d + :Spam Me!", s_Spam,
                settings.name, 1);
                /* This is the server token (which is ugly as sin)
                ** For now, I'll send '1'... Once I find out more about
                ** it, I'll send the proper value.
                */
#endif
#endif

#ifdef IRCD_2_10
	AddUser(s_Spam, "please", settings.name, findserver(settings.name));
#else
	AddUser(s_Spam, "please", settings.name, settings.name);
#endif
}
#endif

#ifdef SPLITSERV
/* Bring up the Split detecting/reporting client if it was enabled */
void init_splitserv()
{

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
        sts("NICK %s 1 1 + %s %s %s :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, settings.name, sp.desc);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d + :%s", SplitServ.nick, SplitServ.user,
                SplitServ.host, 1, sp.desc);
                /* This is the server token (which is ugly as sin)
                ** For now, I'll send '1'... Once I find out more about
                ** it, I'll send the proper value.
                */
#endif
#endif

#ifdef IRCD_2_10
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host,
		findserver(settings.name));
#else
        AddUser(SplitServ.nick, SplitServ.user, SplitServ.host, settings.name);
#endif
}
#endif

/* Bring up the administration service */
void init_admin()
{
	User *u = NULL;

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
        sts("NICK %s 1 1 +io %s %s %s :Administration Service",
		AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#else
#if defined(IRCD_2_10)
        sts("NICK %s 1 %s %s %d +io :Administration Service", AdminServ.nick, AdminServ.user,
                AdminServ.host, 1);
#endif
#endif

#ifdef IRCD_2_10
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host,
		findserver(settings.name));
#else
        AddUser(AdminServ.nick, AdminServ.user, AdminServ.host, settings.name);
#endif

	u = finduser(AdminServ.nick);
	IncreaseOpers(findserver(settings.name));
	u->flags |= UFLAG_ISOPER;
#ifdef OPER_MODES
        u->operflags |= OPER_GLOBAL;
#endif
}

#ifdef STATSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   int i;
   if (netinfo.chan_count)
   {
	for (i = 0; i < netinfo.chan_count; i++)
	{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
			sts(":%s SJOIN %ld %s + :@%s", settings.name, 
			    time(NULL), me.chan_list[i], StatServ.nick);
#else
#if defined(IRCD_2_10)
			/* Here is the only one that we hack ops... */
			sts(":%s NJOIN %s :@%s", settings.name, me.chan_list[i], 
				StatServ.nick);
#endif
#endif
	}
   }
}
#endif

#ifdef SPLITSERV
/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Split
*/
void init_spjoin()
{
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
	sts(":%s SJOIN %ld %s + :@%s", settings.name,
        	time(NULL), sp.join_chan, SplitServ.nick);
#else
#if defined(IRCD_2_10)
        /* Here is the only one that we hack ops... */
        sts(":%s NJOIN %s :@%s", settings.name, sp.join_chan, SplitServ.nick);
#endif
#endif
}
#endif

void CmdLine(int argc, char *argv[])
{
	char c;
	extern char *optarg;
	extern int optind;

	while( (c=getopt(argc, argv, "dvhn")) != -1)
	{
		switch (c)
		{
			case 'd':
				me.cmdflags |= CMD_DEBUG;
				break;
			case 'v':
				PrintVersion();
				exit(0);
				/* NOT REACHED */
				break;
			case 'h':
				PrintHelp();
				exit(0);
				/* NOT REACHED */
				break;
			case 'n':
				me.cmdflags |= CMD_NOFORK;
				break;
		}
	}

	
}

void PrintVersion()
{
	printf("Stats version %s(%s)\nSPATH: %s\n", PATCHLEVEL, SERIALNUM,
		SPATH);
	printf("Stats NOT Started.\n");
}

void PrintHelp()
{
	printf("stats [-n|-d|-h|-v]\n-n Do not fork into the background\n");
	printf("-d Set debugging mode\n-h Print this help screen\n");
	printf("-v Print the version and exit\n");
	printf("Stats NOT Started.\n");
}
