/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: sock.c,v 1.24 2001/09/03 14:21:47 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

fd_set readfds, nullfds;

static int Server_receiveFrom(int, char *);
static void Service_processPacket(char *);

/* Initiate the connection to the server, returns the FD */
int ConnectTo(char *host, int port)
{
	struct hostent *hp;
	struct sockaddr_in sa;
	struct in_addr * in;
	int s;

	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		log("Cannot create socket()");
		return (-1);
	}

        if (IsVHost())
	{
                memset((void *) &sa, '\0', sizeof(sa));
                sa.sin_family = AF_INET;
                if ((hp = gethostbyname (settings.vhost)) == NULL)
		{
			log("Unable to gethostbyname(%s)", settings.vhost);
                        return (-1);
                }
                in = (struct in_addr *) (hp->h_addr_list[0]);
                sa.sin_addr.s_addr = in->s_addr;
                sa.sin_port = 0;
                if (bind(s, (struct sockaddr *) &sa, sizeof(sa)) < 0)
		{
                        close(s);
			log("Unable to bind to VHOST %s", settings.vhost);
                        return (-1);
                }
        }

        if ((hp = gethostbyname (host)) == NULL)
        {
                log("Cannot gethostbyname(%s)", host);
                return (-1);
        }

	memset((void *) &sa, '\0', sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (port);
	memcpy((void *) &sa.sin_addr, (void *) hp->h_addr, hp->h_length);

	if (connect (s, (struct sockaddr *) &sa, sizeof (sa)) < 0)
	{
		close (s);
		log("Cannot connect()");
		return (-1);
	}

	if (IsDebug())
	{
		log("Connected to server(%s) port(%d)", host, port);
	}
	return s;
}

/* Here is where it all happens, get a line from the server
** and send it off to the parse function.  Also this is where
** chk, the timer function, is called.  
*/
void read_loop()
{
	int SelectResult;
	struct timeval TimeOut;
	static char buf[BUFSIZE + 1];

	while (1)
	{
		memset(buf, '\0', BUFSIZE);
		chk();
		FD_ZERO(&readfds);
		TimeOut.tv_sec = 500;
		TimeOut.tv_usec = 0L;
		FD_SET(servsock, &readfds);
		if ((SelectResult = select(FD_SETSIZE, &readfds, &nullfds,
			&nullfds, &TimeOut)) > 0)
		{
			if (!Server_receiveFrom(servsock, buf))
			{
				log("Lost connection to server.");
				return;
			}
			Service_processPacket(buf);
		}
		else
		{
			if (SelectResult == 0)
			{
				log("Lost connection to server.");
				return;
			}
		}
	}
}

/* send a line of text to the server, Append the \n */
void sts(char *fmt,...)
{
	va_list ap;
	char buf[BUFSIZE + 1];

	va_start (ap, fmt);
	vsnprintf (buf, BUFSIZE, fmt, ap);
	buf[BUFSIZE] = '\0';

	if (IsDebug())
	{
		log("SENT: %s", buf);
	}
	strncat(buf, "\n", BUFSIZE);

	netinfo.outb += strlen(buf);

	if ((write(servsock, buf, strlen(buf))) == -1)
	{
		log("Write error.");
		exit(0);
	}

	va_end (ap);
}

/* These two functions were lifted from oswald's NoteServ */
int Server_receiveFrom(int fd, char *buffer)
{
    /* This mustn't be BUFSIZE!!! */
    /* unless buffer is BUFSIZE + 1 */
    int n = read(fd, buffer, BUFSIZE);
    buffer[n] = '\0';
    return n;
}

void Service_processPacket(char *buffer)
{
    char *ptr;
    char buffer2[BUFSIZE * 8 + 1];
    static char tmp[BUFSIZE * 8 + 1];

    while ((ptr = strchr(buffer, '\n')))
    {
        *ptr = '\0';
        if (*(ptr - 1) == '\r')
            *(ptr - 1) = '\0';
        snprintf(buffer2, (BUFSIZE * 8), "%s%s", tmp, buffer);
	buffer2[BUFSIZE * 8] = '\0';
        *tmp = '\0';
        parse(buffer2);
        buffer = ptr + 1;
    }
    if (*buffer)
    {
        strncpy(tmp, buffer, (BUFSIZE * 8) - strlen(tmp));
	tmp[BUFSIZE * 8] = '\0';
    }
}
