/*****************************************
* data.c - Data Manipulation Functions
*
* $Id: split-data.c,v 1.11 2001/09/05 22:10:25 wcampbel Exp $
*****************************************/

#include "config.h"
#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SPLITSERV

void SpInitTables()
{
	int i;

        for (i=0; i<MAXSERVER; i++)
	{
                server_table[i] = NULL;
        }
        map_head = SpNewServer(settings.name, NULL);
        for (i=0; i<MAXSERVER; i++)
	{
                split_table[i] = NULL;
        }
}

SpServer* SpNewServer(char* name, char* uplink)
{
	int i,j;
	SpServer* up = SpFindServer(uplink);
	SpServer* s = malloc(sizeof(Server));
	strncpy(s->name, name, MAXHOST);
	s->uplink = up;
	s->numleaf = 0;
	for (i=0; i<MAXSERVER; i++)
	{
		s->leaflist[i] = NULL;
	}
	if (up)
	{
		up->numleaf++;
		for(j=0; j<MAXSERVER; j++)
		{
			if (!up->leaflist[j])
			{
				up->leaflist[j] = s;
				break;
			}
		}
	}
	else
	{
		log("WARNING: Unable to find uplink: %s for server name: %s",
			uplink, name);
	}
	for (i=0; i<MAXSERVER; i++)
	{
		if (!server_table[i])
		{
			server_table[i] = s;
			break;
		}
	}
	return s;
}

void SpDelServer(char* name)
{
	int i;
	int j;
	SpServer* s = NULL;
	if (name)
	{
		for (j=0; j<MAXSERVER; j++)
		{
			if (server_table[j])
			{
				if (!strcasecmp(name, server_table[j]->name))
				{
					s = server_table[j];
					server_table[j] = NULL;
				}
			}
		}
		if (s)
		{
		    if (s->uplink)
		    {
			s->uplink->numleaf--;

			for (i=0; i<MAXSERVER; i++)
			{
				if (s->uplink->leaflist[i])
				{
					if (!strcasecmp(name, 
					    s->uplink->leaflist[i]->name))
					{
						s->uplink->leaflist[i] = NULL;
					}
				}
			}

                    }
		    else
		    {
                        log("No uplink for server: %s", s->name);
		    }

		    free(s);
		}
		else
		{
		    log("No server table entry for server name: %s", name);
		}
	}
}

SpServer* SpFindServer(char* name)
{
	int i;
	if (name)
	{
		for (i=0; i<MAXSERVER; i++)
		{
			if (server_table[i])
			{
				if (!strcasecmp(server_table[i]->name, name))
				{
					return server_table[i];
				}
			}
		}
	}
	if (IsDebug())
		log("Unable to FindServer for name: %s", name);
	return NULL;
}

void SpNewSplit(char * name)
{
	SpServer* s = malloc(sizeof(SpServer));
	SpServer *t = SpFindSplit(name);
	int i;
	if (t)
	{
		log("Split entry for server name: %s already exists", name);
		return;
	}
	strncpy(s->name, name, MAXHOST);
	s->ts = time(NULL);
	/* None of the other variables are used here */
	for (i=0; i<MAXSERVER; i++)
	{
		if (!split_table[i])
		{
			split_table[i] = s;
			break;
		}
	}
}

void SpDelSplit(char * name)
{
	SpServer* s = NULL;
	int i;
	for (i=0; i<MAXSERVER; i++)
	{
		if (split_table[i])
		{
			if (!strcasecmp(split_table[i]->name, name))
			{
				s = split_table[i];
				split_table[i] = NULL;
				break;
			}
		}
	}
	if (s)
	{
		free(s);
	}
	else
	{
		if (!is_delayed)
			log("Unable to DelSplit server name: %s", name);
	}
}

SpServer* SpFindSplit(char* name)
{
        int i;
        if (name)
	{
                for (i=0; i<MAXSERVER; i++)
                {
                        if (split_table[i])
			{
                                if (!strcasecmp(split_table[i]->name, name))
				{
                                        return split_table[i];
                                }
                        }
                }
        }
	if (IsDebug())
		log("Unable to FindSplit server name: %s", name);
        return NULL;
}

#endif
