/*********************************************
* util.c - Utility functions
*
* $Id: split-util.c,v 1.29 2001/09/13 16:10:47 wcampbel Exp $
*********************************************/

#include "config.h"
#include "stats.h"
#include "struct.h"
#include "proto.h"

#ifdef SPLITSERV

void dump_map()
{
	FILE *fp = fopen(sp.text_name, "w");
	dump_text(fp, map_head, 0);
	fclose(fp);
}
	
void dump_text(FILE *fp, SpServer* s, int level)
{
        int i;
        char* d = dash(level);
        if (s)
	{
                fprintf(fp, "%s %s\n", d, s->name);
                for (i=0; i<MAXSERVER; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        dump_text(fp, s->leaflist[i], level+1);
                                }
                        }
                }
        }
}

void html_map(FILE *fp, SpServer* s, int level)
{
        int i;
        char* d = hdash(level);
        if (s)
	{
                fprintf(fp, "%s <A HREF=\"#%s\">%s</A><BR>\n", d,
			s->name, s->name);
                for (i=0; i<MAXSERVER; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        html_map(fp, s->leaflist[i], level+1);
                                }
                        }
                }
        }
}

/* REWRITE THIS DAMN FUNCTION */
char* dash(int level)
{
        int i;
        static char s[MAXDEPTH * 4 + 1];
        s[0] = '\0';
        if (!level)
	{
                return s;
        }
        s[0] = ' ';
        s[1] = '\0';
        for (i=level - 1; i>=0; i--)
	{
                if (!i)
		{
                        snprintf(s, MAXDEPTH * 4, "%s`-", s);
                }
		else if (i == (level - 1))
		{
                        snprintf(s, MAXDEPTH * 4, "%s    ", s);
                }
		else
		{
                        snprintf(s, MAXDEPTH * 4, "%s|   ", s);
                }
        }
	s[MAXDEPTH * 4] = '\0';
        return s;
}

char* hdash(int level)
{
        int i;
        static char s[MAXDEPTH * 10 + 1];
        s[0] = '\0';
        if (!level)
	{
                return s;
        }
	s[0] = '&';
        s[1] = 'n';
	s[2] = 'b';
	s[3] = 's';
	s[4] = 'p';
	s[5] = ';';
        s[6] = '\0';
        for (i=level - 1; i>=0; i--)
	{
                if (!i)
		{
                        snprintf(s, MAXDEPTH*10, "%s`-", s);
                }
		else if (i == (level - 1))
		{
                        snprintf(s, MAXDEPTH*10, "%s&nbsp;&nbsp;&nbsp;&nbsp;",
				s);
                }
		else
		{
                        snprintf(s, MAXDEPTH*10, "%s|&nbsp;&nbsp;&nbsp;", s);
                }
        }
	s[MAXDEPTH * 10] = '\0';
        return s;
}

void SpFlushServerTree(SpServer *s)
{
        int i;
        if (s)
	{
                for (i=0; i<MAXSERVER; i++)
		{
                        if(s->leaflist[i])
			{
                                SpFlushServerTree(s->leaflist[i]);
                                s->leaflist[i]=NULL;
                        }
                }
                /* Damn, extra calls to findserver :( */
                SpDelServer(s->name);
                /* Don't add things to the split table */
        }
}

/* Send a message to a user from a user */
void spprivmsg(char *to, char *from, char *fmt, ...)
{
        va_list ap;
        char buf[BUFSIZE], buf2[BUFSIZE];

        va_start(ap, fmt);
        vsnprintf(buf2, sizeof(buf2), fmt, ap);
        if (is_channel(to))
	{
            if (IsSPChanmsg())
                snprintf(buf, BUFSIZE, ":%s PRIVMSG %s :%s", from, to, buf2);
            else
                snprintf(buf, BUFSIZE, ":%s NOTICE %s :%s", from, to, buf2);
        }
        else
	{
            if (IsSPPrivmsg())
                snprintf(buf, BUFSIZE, ":%s PRIVMSG %s :%s", from, to, buf2);
            else
                snprintf(buf, BUFSIZE, ":%s NOTICE %s :%s", from, to, buf2);
        }
        sts("%s", buf);
        va_end(ap);
}

void PrintMap(char* target, SpServer* s, int level)
{
        int i;
        struct timeval tv;
        char* d = dash(level);
        if (s)
	{
                spprivmsg(target, SplitServ.nick, "%s %s", d, s->name);
                tv.tv_usec = 750000; /* 3/4 a second delay */
                tv.tv_sec = 0;       /* ignore this */
                select(1, NULL, NULL, NULL, &tv);
                for (i=0; i<MAXSERVER; i++)
		{
                        if (s->leaflist[i])
			{
                                if (s->leaflist[i]->name)
				{
                                        PrintMap(target, s->leaflist[i],
                                                (level+1));
                                }
                        }
                }
        }
}

void PrintSplit(char* target)
{
        int i;
        int count = 0;
        char * td;
        struct timeval tv;
        time_t now = time(NULL);

        /* Everything in the Split List will have been created with
           NewSplit, so it will have a valid time stamp */

        for (i=0; i<MAXSERVER; i++)
	{
                if (split_table[i])
		{
                        td = timediff(now - split_table[i]->ts);
                        count++;
                        spprivmsg(target, SplitServ.nick, 
				"SPLIT SERVER: %s for %s",
                                split_table[i]->name, td);
                        tv.tv_usec = 750000; /* 3/4 a second delay */
                        tv.tv_sec = 0;       /* ignore this */
                        select(1, NULL, NULL, NULL, &tv);
                }
        }
        if (!count)
	{
                spprivmsg(target, SplitServ.nick, "NO SPLITS");
        }
}

void PrintDump(char * target)
{
        int i,j;
        char leaves[BUFSIZE] = "";
        struct timeval tv;
        for (i=0; i<MAXSERVER; i++)
	{
                leaves[0] = '\0';
                if (server_table[i])
		{
                        for (j=0; j<MAXSERVER; j++)
			{
                                if (server_table[i]->leaflist[j])
				{
                                        snprintf(leaves, BUFSIZE, "%s %s",
					  leaves,
                                          server_table[i]->leaflist[j]->name);
                                }
                        }
                        spprivmsg(target, SplitServ.nick,
				"Server: %s Leaves:%s", server_table[i]->name,
                                (!strcmp(leaves, "")) ?  " None" : leaves);
                        tv.tv_usec = 750000; /* 3/4 a second delay */
                        tv.tv_sec = 0;       /* ignore this */
                        select(1, NULL, NULL, NULL, &tv);
                }
        }
}

void Update(char *origin)
{
        if (IsSPEnableText())
        {
                dump_map();
                spprivmsg(origin, SplitServ.nick, "MAP file updated");
            return;
        }
        spprivmsg(origin, SplitServ.nick, "MAP Text Dump Disabled");
        return;
}

void Add(char * origin, char * serv)
{
        SpServer *s = NULL;

        s = SpFindServer(serv);
        if (s)
	{
           spprivmsg(origin, SplitServ.nick, 
		"Can't add a split for an existing server");
           return;
        }
        s = SpFindSplit(serv);
        if (s)
	{
           spprivmsg(origin, SplitServ.nick, 
		"Can't add a split for an existing split");
           return;
        }
        SpNewSplit(serv);
        spprivmsg(sp.join_chan, SplitServ.nick, "ADDED SPLIT: %s", serv);
        splog("Added split %s by %s", serv, origin);
        return;
}

void Forget(char * origin, char * serv)
{
        int i;
        SpServer * s = NULL;

        for (i=0; i<MAXSERVER; i++)
	{
            if (split_table[i])
	    {
                if (!strcasecmp(split_table[i]->name, serv))
		{
                    spprivmsg(origin, SplitServ.nick,
			"Removed from the Split Table",
                        split_table[i]->name);
                    s = split_table[i];
                    split_table[i] = NULL;
                    spprivmsg(sp.join_chan, SplitServ.nick, 
			"FORGOT SPLIT: %s", s->name);
                    splog("Forgot split %s by %s", s->name, origin);
                    free(s);
                    return;
                }
            }
        }

        spprivmsg(origin, SplitServ.nick, 
		"%s NOT found in the Split Table", serv);
        return;
}

void spprivmsg_list(char *to, char *from, const char **text)
{
        while (*text)
	{
                if (**text)
                        spprivmsg(to, from, "%s", *text);
                else
                        spprivmsg(to, from, " ", *text);
                text++;
        }
}

#endif
