/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: stats.c,v 1.18 2001/09/03 22:48:00 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

Stats *shead;
static Stats *new_stats(const char *);

/* print a period of seconds into human readable format */
char *timediff(time_t u)
{
	static char buf[TIMEBUF];

	if (u > 86400)
		snprintf(buf, TIMEBUF, "%ld day%s, %02ld:%02ld",
			u/86400, (u/86400 == 1) ? "" : "s",
			(u/3600) % 24, (u/60) % 60);
	else if (u > 3600)
		snprintf(buf, TIMEBUF, "%ld hour%s, %ld minute%s",
			u/3600, u/3600==1 ? "" : "s", (u/60) % 60,
			(u/60)%60 == 1 ? "" : "s");
	else
		snprintf(buf, TIMEBUF,
			"%ld minute%s, %ld second%s",
			u/60, u/60 == 1 ? "" : "s",
			u%60, u%60 == 1 ? "" : "s");

	return buf;
}

void IncreaseOpers(Server *s)
{
	if (IsDebug())
	{
		log("Increasing OperCount for %s", s->name);
	}
	s->opers++;
	netinfo.opers++;
	if (s->opers > s->stats->maxopers)
	{
		s->stats->maxopers = s->opers;
		s->stats->t_maxopers = time(NULL);
	}
	if (netinfo.opers > netinfo.maxopers)
	{
		netinfo.maxopers = netinfo.opers;
		netinfo.t_maxopers = time(NULL);
	}
	if (netinfo.opers > daily.opers)
	{
		daily.opers = netinfo.opers;
		daily.t_opers = time(NULL);
	}
}

void IncreaseUsers(Server *s)
{
	if (IsDebug())
	{
		log("Increasing UserCount for %s", s->name);
	}
	s->users++;
	netinfo.users++;
	if (s->users > s->stats->maxusers)
	{
		s->stats->maxusers = s->users;
		s->stats->t_maxusers = time(NULL);
	}
	if (netinfo.users > netinfo.maxusers)
	{
		netinfo.maxusers = netinfo.users;
		netinfo.t_maxusers = time(NULL);
	}
	if (netinfo.users > daily.users)
	{
		daily.users = netinfo.users;
		daily.t_users = time(NULL);
	}
}

void IncreaseServers()
{
	if (IsDebug())
	{
		log("Increasing server count");
	}
	netinfo.servers++;
	if (netinfo.servers > netinfo.maxservers)
	{
		netinfo.maxservers = netinfo.servers;
		netinfo.t_maxservers = time(NULL);
	}
	if (netinfo.servers > daily.servers)
	{
		daily.servers = netinfo.servers;
		daily.t_servers = time(NULL);
	}
}

/* Create a new Stats struct for a server we haven't seen before */
static Stats *new_stats(const char *name)
{
	Stats *s = calloc(sizeof(Stats), 1);

	if (IsDebug())
	{
		log("new_stats(%s)", name);
	}

	if (!s)
	{
		log("Out of memory.");
		exit(0);
	}

	memcpy(s->name, name, MAXHOST);
	s->numsplits = 0;
	s->maxusers = 0L;
	s->t_maxusers = time(NULL);
	s->t_maxopers = time(NULL);
	s->maxopers = 0;
	s->lastseen = time(NULL);
	s->starttime = time(NULL);

	if (!shead)
	{
		shead = s;
		shead->next = NULL;
	}
	else
	{
		s->next = shead;
		shead = s;
	}
	return s;
}

void AddStats(Server *s)
{
	Stats *st = findstats(s->name);

	if (IsDebug())
	{
		log("AddStats(%s)", s->name);
	}

	if (!st)
	{
		st = new_stats(s->name);
		s->stats = st;
	}
	else
	{
		s->stats = st;
		st->lastseen = time(NULL);
	}
}

Stats *findstats(char *name)
{
	Stats *t;
	if (IsDebug())
	{
		log("findstats(%s)", name);
	}
	for (t = shead; t; t = t->next)
		if (!strcasecmp(name, t->name))
			return t;
	return NULL;
}

void SaveStats()
{
	FILE *fp = fopen(STATS_DB, "w");
	Stats *s;

	if (!fp)
	{
		log("Unable to open %s for writing.", STATS_DB);
		return;
	}

	for (s = shead; s; s = s->next)
	{
		if (IsDebug())
		{
			log("Writing statistics to database for %s", s->name);
		}
		fprintf(fp, "%s %d %ld %ld %d %ld %ld %ld %d %d\n", s->name,
			s->numsplits, s->maxusers, s->t_maxusers, s->maxopers,
			s->t_maxopers, s->lastseen, s->starttime, s->operkills,
			s->serverkills);
	}
	fclose(fp);

	if ((fp = fopen(NSTATS_DB, "w")) == NULL)
	{
		log("Unable to open %s for writing.", NSTATS_DB);
		return;
	}
	fprintf(fp, "%d %ld %d %ld %ld %ld\n", netinfo.maxopers,
		netinfo.maxusers, netinfo.maxservers, netinfo.t_maxopers,
		netinfo.t_maxusers, netinfo.t_maxservers);
	fclose(fp);
}

/* NOTE that if the format of the statistics database is incorrect.  Stats
** WILL crash.  I hope you kept backups...
*/
void LoadStats()
{
	FILE *fp = fopen(NSTATS_DB, "r");
	Stats *s;
	char buffer[BUFSIZE];
	char *name, *numsplits, *maxusers, *t_maxusers,
		*maxopers, *t_maxopers, *lastseen, *starttime,
		*operkills, *serverkills;

	if (fp)
	{
		while (fgets(buffer, BUFSIZE, fp))
		{
			netinfo.maxopers = atoi(strtok(buffer, " "));
			netinfo.maxusers = atol(strtok(NULL, " "));
			netinfo.maxservers = atoi(strtok(NULL, " "));
			netinfo.t_maxopers = atoi(strtok(NULL, " "));
			netinfo.t_maxusers = atol(strtok(NULL, " "));
			netinfo.t_maxservers = atoi(strtok(NULL, " "));
		}
		fclose(fp);
	}

	if ((fp = fopen(STATS_DB, "r")) == NULL)
		return;

	memset(buffer, '\0', BUFSIZE);
	while (fgets(buffer, BUFSIZE, fp))
	{
		s = smalloc(sizeof(Stats));
		name = strtok(buffer, " ");
		numsplits = strtok(NULL, " ");
		maxusers = strtok(NULL, " ");
		t_maxusers = strtok(NULL, " ");
		maxopers = strtok(NULL, " ");
		t_maxopers = strtok(NULL, " ");
		lastseen = strtok(NULL, " ");
		starttime = strtok(NULL, " ");
		operkills = strtok(NULL, " ");
		serverkills = strtok(NULL, " ");

		memcpy(s->name, name, MAXHOST);
		s->numsplits = atoi(numsplits);
		s->maxusers = atol(maxusers);
		s->t_maxusers = atol(t_maxusers);
		s->maxopers = atoi(maxopers);
		s->t_maxopers = atol(t_maxopers);
		s->lastseen = atol(lastseen);
		s->starttime = atol(starttime);
		s->operkills = atoi(operkills);
		s->serverkills = atoi(serverkills);

		if (IsDebug())
		{
			log("LoadStats(): Loaded statistics for %s", s->name);
		}
		if (!shead)
		{
			shead = s;
			shead->next = NULL;
		}
		else
		{
			s->next = shead;
			shead = s;
		}
	}
	fclose(fp);
}
