/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: statserv.c,v 1.66 2001/09/12 15:27:19 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "help.h"
#include "patchlevel.h"

#ifdef STATSERV

extern const char version_date[], version_time[];
static char errstr[BUFSIZE];

int s_server(char *, int, char **);
int s_map(char *, int, char **);
#ifdef TRACK_VERSION
int s_version_map(char *, int, char **);
#endif
int s_uptime(char *, int, char **);
int s_netstats(char *, int, char **);
int s_daily(char *, int, char **);
int s_stats(char *, int, char **);
int s_tld(char *, int, char **);
int s_tld_map(char *, int, char **);
int s_operlist(char *, int, char **);
int s_join(char *, int, char **);
int s_cping(char *, int, char **);
int s_cversion(char *, int, char **);
int s_update(char *, int, char **);
int s_url(char *, int, char **);
int s_status(char *, int, char **);
int s_help(char *, int, char **);

static struct MsgTable stat_table[] =
{
	{ "HELP",	0,	s_help },
	{ "SERVER",	0,	s_server },
	{ "MAP",	0,	s_map },
#ifdef TRACK_VERSION
	{ "VERSIONMAP",	0,	s_version_map },
#endif
	{ "UPTIME",	0,	s_uptime },
	{ "NETSTATS",	0,	s_netstats },
	{ "DAILY",	0,	s_daily },
	{ "TLD",	0,	s_tld },
	{ "TLDMAP",	0,	s_tld_map },
	{ "OPERLIST",	0,	s_operlist },
	{ "STATS",	0,	s_stats },
	{ "JOIN",	0,	s_join },
	{ "\001PING",	0,	s_cping },
	{ "\001VERSION\001",	0,	s_cversion },
	{ "UPDATE",	0,	s_update },
	{ "URL",	0,	s_url },
	{ "STATUS",	0,	s_status },
	{ (char *)0,	0,	(int (*)()) 0 }
};

static struct HelpTable help_table[] =
{
        { "SERVER",     ss_server_help,         0 },
        { "MAP",        ss_map_help,            0 },
        { "NETSTATS",   ss_netstats_help,       0 },
        { "DAILY",      ss_daily_help,          0 },
        { "TLD",        ss_tld_help,            0 },
        { "TLDMAP",     ss_tld_map_help,        0 },
        { "OPERLIST",   ss_operlist_help,       0 },
        { "STATS",      ss_stats_help,          1 },
        { "JOIN",       ss_join_help,           1 },
        { "UPDATE",     ss_update_help,         1 },
        { "URL",        ss_url_help,            0 },
        { "STATUS",     ss_status_help,         0 },
        { "UPTIME",     ss_uptime_help,         0 },
#ifdef TRACK_VERSION
        { "VERSIONMAP", ss_version_map_help,    0 },
#endif
        { (char *)0,    (const char **)0,       0 }
};


void statserv(char *origin, char *param)
{
	char *parv[MAXPARA];
	int parc = 0;
	User *u = NULL;
	int i;
	int status;
	static char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* Clear the error string */
	errstr[0] = '\0';

	strncpy(buf, param, BUFSIZE);

	if (param)
		parc = msgtoken(param, parv);

	if (origin)
		u = finduser(origin);
	else
		return;

        if (!u)
	{
                log("Unable to finduser %s (statserv)", origin);
                return;
        }

	netinfo.requests++;

        if (flood(u))
                return;

	if (!parc)
		return;

	if (parv[0][0] != '\001')
		log("%s received message from %s: %s", StatServ.nick, 
			origin, buf);

        if (IsOnlyOpers() && !IsUIsOper())
	{
		if (parv[0][0] != '\001')
		{
                	privmsg(u->nick, StatServ.nick,
                       		 "This service is only available to IRCops.");
                	return;
		}
        }

	/* All the preliminary junk is done, now we have to go through 
	** the tables and call the handler for that command
	*/
        for (i=0; stat_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], stat_table[i].name))
                {
                    status = stat_table[i].handler(u->nick, parc, parv);
                    if (status == 1)
                        log("Error with statserv command %s: %s", parv[0],
			(errstr) ? errstr : "No error message");
		    /* Matched a value, done, then return */
		    return;
                }
        }
	/* If we get here, we did not get a valid command...
	** Make sure not to say anything to the user if it was a 
	** CTCP that was not matched
	*/
	if (*parv[0] == '\001')
		return;
	privmsg(u->nick, StatServ.nick, "Invalid command: %s", parv[0]);
	return;
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

int s_help(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	int i;

	if (!parv[1])
	{
		privmsg_list(u->nick, StatServ.nick, ss_help);
		if (u->myuser != NULL)
			privmsg_list(u->nick, StatServ.nick, ss_myuser_help);
		return 0;
	}

	for (i = 0; help_table[i].helpmsg; i++)
	{
		if (!strcasecmp(parv[1], help_table[i].name))
                {
			if (help_table[i].myuonly)
			{
				if (u->myuser)
					privmsg_list(u->nick, StatServ.nick,
						help_table[i].helpmsg);
				else
					privmsg(u->nick, StatServ.nick,
						"Permission Denied.");
			}
			else
			{
				privmsg_list(u->nick, StatServ.nick,
					help_table[i].helpmsg);
			}

			return 0;
		}
	}

	privmsg(u->nick, StatServ.nick, "Unknown Help Topic: \2%s\2", parv[1]);
	snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
	return 1;
}

int s_server(char *origin, int parc, char *parv[])
{
	Server *s = NULL;
	Stats *st = NULL;

	if (!parv[1])
	{
		privmsg(origin, StatServ.nick,
			"Syntax: /msg %s SERVER <server name>", StatServ.nick);
		strcpy(errstr, "Missing parameter to SERVER");
		return 1;
	}

	s = findserver(parv[1]);
	st = findstats(parv[1]);

	if (!st)
	{
		privmsg(origin, StatServ.nick, "No information available on %s",
			parv[1]);
		return 0;
	}

        /* If we want services to be hidden, catch it here */
        if (IsHide() && (!strcasecmp(parv[1], settings.services_name)))
        {
                privmsg(origin, StatServ.nick,
                    "No information available on %s (hidden server)", parv[1]);
                return 0;
        }

        privmsg(origin, StatServ.nick, "Statistics for %s since %s", parv[1],
                sftime(st->starttime));
        if (!s)
                privmsg(origin, StatServ.nick, "Server Last Seen:  %s",
                        sftime(st->lastseen));
        if (s)
                privmsg(origin, StatServ.nick, "Current Users: %-3ld (%.0f%%)",
			s->users,
                        (float)s->users / (float)netinfo.users * 100);
        privmsg(origin, StatServ.nick, "Maximum Users: %-3ld %s", st->maxusers,
                sftime(st->t_maxusers));
        if (s)
                privmsg(origin, StatServ.nick, "Current Opers: %d", s->opers);
        privmsg(origin, StatServ.nick, "Maximum Opers: %-3d %s", st->maxopers,
                sftime(st->t_maxopers));
        privmsg(origin, StatServ.nick, "  IRCop Kills: %d", st->operkills);
        privmsg(origin, StatServ.nick, " Server Kills: %d", st->serverkills);
#ifdef LAG_CHECK
        if (s)
	{
        	privmsg(origin, StatServ.nick, "  Lowest Ping: %-3d %s",
			s->lowest_ping, sftime(s->t_lowest_ping));
        	privmsg(origin, StatServ.nick, " Highest Ping: %-3d %s",
			s->highest_ping, sftime(s->t_highest_ping));
        	privmsg(origin, StatServ.nick, " Current Ping: %d", s->ping);
        }
#endif
        if (st->numsplits >= 1)
        privmsg(origin, StatServ.nick,
		"%s has split from the network %d time%s.",
                st->name, st->numsplits, (st->numsplits == 1) ? "" : "s");
        else
        privmsg(origin, StatServ.nick, "%s has never split from the network.",
                st->name);
#ifdef TRACK_VERSION
	if (s)
	{
	        if (s->version[0])
        	        privmsg(origin, StatServ.nick, "Version: %s",
				s->version);
	        else
        	        privmsg(origin, StatServ.nick, "Version Unknown");
	}
#endif
        privmsg(origin, StatServ.nick, "End of Statistics.");

	return 0;
}

#ifdef TRACK_VERSION
int s_version_map(char *origin, int parc, char *parv[])
{
        Server *s = NULL;
        int i;

        privmsg(origin, StatServ.nick,
                "[\2Server\2]                      [\2Version\2]");
        for (i = 0; i < S_TABLE_SIZE; i++)
	{
                for (s = serverlist[i]; s; s = s->next)
		{
                    if (IsHide())
		    {
                        if (strcasecmp(s->name, settings.services_name))
			{
                            privmsg(origin, StatServ.nick,
                                "%-30s %s", s->name,
				(s->version && s->version[0]) ?  s->version :
				"Unknown Version");
                        }
                    }
		    else
		    {
                        privmsg(origin, StatServ.nick, "%-30s %s",
                                s->name, (s->version && s->version[0]) ?
				s->version : "Unknown Version");
                    }
                }
        }
        privmsg(origin, StatServ.nick, "End of Version Map.");

	return 0;
}
#endif

int s_map(char *origin, int parc, char *parv[])
{
        Server *s;
        int i;

#ifdef LAG_CHECK
        privmsg(origin, StatServ.nick, "Network Map [\2NAME\2] [\2USERS/MAX\2] "
                "[\2OPERS/MAX\2] [\2PERCENT\2] [\2LAG\2]");
#else
        privmsg(origin, StatServ.nick, "Network Map [\2NAME\2] [\2USERS/MAX\2] "
                "[\2OPERS/MAX\2] [\2PERCENT\2]");
#endif
        for (i = 0; i < S_TABLE_SIZE; i++)
	{
                for (s = serverlist[i]; s; s = s->next)
		{
                    if (IsHide())
		    {
                        /* meaning if the server does NOT match the services
                        ** server, therefore show it
                        */
                        if (strcasecmp(s->name, settings.services_name))
			{
                            /* I should also check for juped servers too -
                            ** servers that have services as an uplink
                            */
#ifdef LAG_CHECK
#ifdef LARGE_NET
                            privmsg(origin, StatServ.nick,
                                "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%] %d",
#else
                            privmsg(origin, StatServ.nick,
                                "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",
#endif /* LARGE_NET */
#else
#ifdef LARGE_NET
                            privmsg(origin, StatServ.nick,
                                "%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%]",
#else
                            privmsg(origin, StatServ.nick,
                                "%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%]",
#endif /* LARGE_NET */
#endif /* LAG_CHECK */
                                s->name, s->users, s->stats->maxusers,
                                s->opers, s->stats->maxopers,
                                (float) s->users / (float) netinfo.users * 100
#ifdef LAG_CHECK
                                ,s->ping);
#else
                                );
#endif
                        }
                    }
		    else
		    {
#ifdef LAG_CHECK
#ifdef LARGE_NET
                            privmsg(origin, StatServ.nick,
				"%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%] %d",
#else
                            privmsg(origin, StatServ.nick,
				"%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%] %d",
#endif /* LARGE_NET */
#else
#ifdef LARGE_NET
                            privmsg(origin, StatServ.nick,
				"%-30s [%3d/%3d] [%2d/%2d] [%2.0f%%]",
#else
                            privmsg(origin, StatServ.nick,
				"%-30s [%2d/%2d] [%2d/%2d] [%2.0f%%]",
#endif /* LARGE_NET */
#endif /* LAG_CHECK */
                                s->name, s->users, s->stats->maxusers,
                                s->opers, s->stats->maxopers,
                                (float) s->users / (float) netinfo.users * 100
#ifdef LAG_CHECK
                                ,s->ping);
#else
                                );
#endif
                    }
                }
        }
#ifdef LARGE_NET
        privmsg(origin, StatServ.nick, "%-30s   [%4d/%4d] [%3d/%3d] [100%%] x",
#else
        privmsg(origin, StatServ.nick, "%-30s   [%2d/%2d] [%2d/%2d] [100%%] x",
#endif
                 "\2Totals:\2", netinfo.users, netinfo.maxusers, netinfo.opers, netinfo.maxopers);
        privmsg(origin, StatServ.nick, "End of Map.");

	return 0;
}

int s_status(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        privmsg(origin, StatServ.nick, "Statistics Status:");
        privmsg(origin, StatServ.nick, "Reconnect Time: %d", settings.r_time);
        privmsg(origin, StatServ.nick, "Extreme Lag Notices: %s",
                IsLag() ? "Enabled" : "Disabled");
        if (IsLag())
                privmsg(origin, StatServ.nick,
			"Lag notices sent for lag greater than %d seconds",
			settings.lag_time);
        privmsg(origin, StatServ.nick, "Oper Only: %s",
                IsOnlyOpers() ? "Enabled" : "Disabled");
        privmsg(origin, StatServ.nick, "HTML Dump: %s",
                IsHtml() ? "Enabled" : "Disabled");
        if (IsHtml())
	{
                privmsg(origin, StatServ.nick, "HTML File: %s", settings.html);
                privmsg(origin, StatServ.nick,
			"HTML File created every %d seconds", settings.html_time);
                privmsg(origin, StatServ.nick, "URL: %s", settings.url);
        }
        privmsg(origin, StatServ.nick, "Hide Services: %s",
                (IsHide()) ? "Enabled" : "Disabled");
        if (IsHide())
                privmsg(origin, StatServ.nick, "Services Name: %s",
                        settings.services_name);
        /* settings.name, settings.desc, settings.uplink all redundant, as they can be seen
        ** from LINKS or from whois StatServ.nick
        */
        privmsg(origin, StatServ.nick, "Set to oper: %s",
                IsSetOper() ? "Enabled" : "Disabled");

    if (u->myuser) {
#ifdef SPAMSERV
	privmsg(origin, StatServ.nick, "Spam Service: Enabled");
#else
	privmsg(origin, StatServ.nick, "Spam Service: Disabled");
#endif
#ifdef ENABLE_JUPE
        privmsg(origin, StatServ.nick, "JUPE: Enabled");
#endif
#ifdef ENABLE_QUOTE
        privmsg(origin, StatServ.nick, "QUOTE: Enabled");
#endif
        if (settings.flood)
	{
                privmsg(origin, StatServ.nick,
                        "Flood Control: %d requests in %d seconds",
			settings.flood, settings.flood_time);
                privmsg(origin, StatServ.nick, "Flood Action: %s",
                        IsFlood() ? "IGNORE" : "KILL");
        }
    }

    return 0;
}

int s_uptime(char *origin, int parc, char *parv[])
{
        privmsg(origin, StatServ.nick, "Uptime and Memory Statistics:");
        privmsg(origin, StatServ.nick, "Statistics up %s",
		timediff(time(NULL) - me.t_start));
        privmsg(origin, StatServ.nick, "Entries in User Hash: %ld",
		netinfo.users);
        privmsg(origin, StatServ.nick, "Entries in Server Hash: %d",
		netinfo.servers);
        privmsg(origin, StatServ.nick, "Statistic Requests: %d",
		netinfo.requests);
        privmsg(origin, StatServ.nick, "User Memory Usage: %d bytes",
                sizeof(User) * netinfo.users);
        privmsg(origin, StatServ.nick, "Server Memory Usage: %d bytes",
                sizeof(Server) * netinfo.servers);
        privmsg(origin, StatServ.nick, "End of Information.");

	return 0;
}

int s_netstats(char *origin, int parc, char *parv[])
{
        privmsg(origin, StatServ.nick, "Network Statistics:");
        privmsg(origin, StatServ.nick, "   Current Users: %ld", netinfo.users);
        privmsg(origin, StatServ.nick, "   Maximum Users: %-2ld %s",
		netinfo.maxusers, sftime(netinfo.t_maxusers));
        privmsg(origin, StatServ.nick, "   Current Opers: %d", netinfo.opers);
        privmsg(origin, StatServ.nick, "   Maximum Opers: %-2d %s",
		netinfo.maxopers, sftime(netinfo.t_maxopers));
        privmsg(origin, StatServ.nick, "   Users Set Away: %-2d", netinfo.away);
        privmsg(origin, StatServ.nick, "   Current Servers: %d",
		netinfo.servers);
        privmsg(origin, StatServ.nick, "   Maximum Servers: %-2d %s",
		netinfo.maxservers, sftime(netinfo.t_maxservers));
        privmsg(origin, StatServ.nick, "End of Information.");

	return 0;
}

int s_daily(char *origin, int parc, char *parv[])
{
        privmsg(origin, StatServ.nick, "Daily Network Statistics:");
        privmsg(origin, StatServ.nick, " ");
        privmsg(origin, StatServ.nick, "  Maximum Servers: %-2d %s",
		daily.servers, sftime(daily.t_servers));
        privmsg(origin, StatServ.nick, "  Maximum Users: %-2d %s", daily.users,
                sftime(daily.t_users));
        privmsg(origin, StatServ.nick, "  Maximum Opers: %-2d %s", daily.opers,
                sftime(daily.t_opers));
        privmsg(origin, StatServ.nick,
		"All daily statistics are reset at midnight.");
        privmsg(origin, StatServ.nick, "End of Information.");

	return 0;
}

int s_tld(char *origin, int parc, char *parv[])
{
        TLD *tmp;
	char *tld = parv[1];

        if (!tld)
	{
                privmsg(origin, StatServ.nick, "Syntax: /msg %s TLD <tld>", 
			StatServ.nick);
                privmsg(origin, StatServ.nick, 
			"For additional help, /msg %s HELP", StatServ.nick);
		strcpy(errstr, "Missing parameter in TLD");
                return 1;
        }

        if (*tld == '*')
                tld++;
        if (*tld == '.')
                tld++;

	/* Catch it when it's just * or . */
	if (!(*tld))
	{
		privmsg(origin, StatServ.nick, "Syntax: /msg %s TLD <tld>",
			StatServ.nick);
		privmsg(origin, StatServ.nick,
			"For additional help, /msg %s HELP", StatServ.nick);
		strcpy(errstr, "Missing parameter in TLD");
		return 1;
	}

        tmp = findtld(tld);

        if (!tmp)
                privmsg(origin, StatServ.nick,
			"Top Level Domain \2%s\2 does not exist.", tld);
        else
                privmsg(origin, StatServ.nick, "%s (%s)", tmp->country,
			tmp->tld);

	return 0;
}

int s_tld_map(char *origin, int parc, char *parv[])
{
        TLD *t;
	int i;

        privmsg(origin, StatServ.nick, "Top Level Domain Statistics:");
	privmsg(origin, StatServ.nick, "   TLD Number / Percent  Country");
	privmsg(origin, StatServ.nick, "   --- ------   -------  -------");
#if 0
        for (t = tldhead; t; t = t->next)
	{
                if (t->users != 0)
                	privmsg(origin, StatServ.nick,
			"%9s \2%3d\2 (%2.0f%%) -> %s", t->tld, t->users,
			(float)t->users / (float)netinfo.users * 100,
			t->country);
        }
#endif
        for (i = 0; i < T_TABLE_SIZE; i++)
        {
                for(t = tldlist[i]; t; t = t->next)
                {
			if (t->users != 0)
				privmsg(origin, StatServ.nick,
				"%9s \2%3d\2 (%2.0f%%) -> %s", t->tld,
				t->users,
				(float)t->users / (float)netinfo.users * 100,
				t->country);
                }
        }
        privmsg(origin, StatServ.nick, "End of Information.");

	return 0;
}

int s_operlist(char *origin, int parc, char *parv[])
{
        int i, j = 0;
        int away = 0;
        User *u;
	char *server = parv[2];

        if (parv[1] && !strcasecmp(parv[1], "NOAWAY"))
	{
                away = 1;
        }
        if (!away && parv[1] && strchr(parv[1], '.'))
                server = parv[1];

        privmsg(origin, StatServ.nick, "On-Line IRCops:");

        for (i = 0; i < U_TABLE_SIZE; i++)
	{
                for (u = userlist[i]; u; u = u->next)
		{
                        if (away && IsUIsAway())
                                continue;
                        if (IsHide() &&
                            !strcasecmp(u->server->name,
					settings.services_name))
                                continue;
                        if (!IsUIsOper())
                                continue;
                        if (!server)
			{
                                if (!IsUIsOper())
					continue;
                                j++;
                                privmsg(origin, StatServ.nick, 
#ifdef OPER_MODES
					"[%2d] [%c] %-15s %s", j, opermode(u),
					u->nick,
#else
					"[%2d] %-15s %s",j, u->nick,
#endif
                                        u->server->name);
                                continue;
                        }
			else
			{
                                if (strcasecmp(server, u->server->name))
					continue;
                                if (!IsUIsOper()) continue;
                                j++;
				privmsg(origin, StatServ.nick,
#ifdef OPER_MODES
					"[%2d] [%c] %-15s %s", j, opermode(u),
#else
					"[%2d] %-15s %s", j,
#endif
					u->nick, u->server->name);
                                continue;
                        }
                }
        }
        privmsg(origin, StatServ.nick, "End of Listing.");

	return 0;
}

int s_stats(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *cmd = parv[1];
	char *arg = parv[2];
	char *arg2 = parv[3];
        Stats *st;

        if (!u->myuser)
	{
                log("Access Denied (STATS) to %s", u->nick);
                privmsg(u->nick, StatServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!cmd)
	{
                privmsg(u->nick, StatServ.nick,
			"Syntax: /msg %s STATS [DEL|LIST|COPY]",
                        StatServ.nick);
                privmsg(u->nick, StatServ.nick,
			"For additional help, /msg %s HELP", StatServ.nick);
		strcpy(errstr, "Missing parameter in STATS");
                return 1;
        }
        if (!strcasecmp(cmd, "LIST"))
	{
                int i = 1;
                privmsg(u->nick, StatServ.nick, "Statistics Database:");
                for (st = shead; st; st = st->next)
		{
                        privmsg(u->nick, StatServ.nick, "[%-2d] %s", i,
			st->name);
                        i++;
                }
                privmsg(u->nick, StatServ.nick, "End of List.");
                log("%s requested STATS LIST.", u->nick);
        }
	else if (!strcasecmp(cmd, "DEL"))
	{
                Stats *m = NULL;
                if (!arg)
		{
                        privmsg(u->nick, StatServ.nick,
				"Syntax: /msg %s STATS DEL <name>",
				StatServ.nick);
                        privmsg(u->nick, StatServ.nick,
				"For additonal help, /msg %s HELP",
				StatServ.nick);
			strcpy(errstr, "Missing parameter in STATS");
                        return 1;
                }
                st = findstats(arg);
                if (!st)
		{
                        privmsg(u->nick, StatServ.nick,
				"%s is not in the database!", arg);
			snprintf(errstr, BUFSIZE, "Unknown server: %s", arg);
                        return 1;
                }
                for (st = shead; st; st = st->next)
		{
                        if (!strcasecmp(arg, st->name))
                                break;
                        m = st;
                }
                if (m)
                        m->next = st->next;
                else
                        shead = st->next;
                free(st);
                privmsg(u->nick, StatServ.nick, "Removed %s from the database.",
			arg);
                log("%s requested STATS DEL %s", u->nick, arg);
        }
	else if (!strcasecmp(cmd, "COPY"))
	{
                Server *s;

                if (!arg || !arg2)
		{
                        privmsg(u->nick, StatServ.nick,
				"Syntax: /msg %s STATS COPY <name> <newname>",
				StatServ.nick);
			strcpy(errstr, "Missing parameter in STATS");
                        return 1;
                }
                st = findstats(arg2);
                if (st)
                        free(st);

                st = findstats(arg);
                if (!st)
		{
                        privmsg(u->nick, StatServ.nick,
				"No entry in the database for %s", arg);
			snprintf(errstr, BUFSIZE, "Unknown stats entry: %s",
				arg);
                        return 1;
                }
                s = findserver(arg);
                if (s)
		{
                        privmsg(u->nick, StatServ.nick, "Server %s is online!",
				arg);
			snprintf(errstr, BUFSIZE, "Server %s is online", arg);
                        return 1;
                }
                s = NULL;
                memcpy(st->name, arg2, sizeof(st->name));
                s = findserver(st->name);
                if (s)
                        s->stats = st;
                privmsg(u->nick, StatServ.nick,
			"Moved database entry for %s to %s", arg, arg2);
                log("%s requested STATS COPY %s -> %s", u->nick, arg, arg2);
        }
	else
	{
                privmsg(u->nick, StatServ.nick, "Invalid Argument.");
                privmsg(u->nick, StatServ.nick, "For help, /msg %s HELP",
			StatServ.nick);
		strcpy(errstr, "Invalid argument in STATS");
		return 1;
        }

	return 0;
}

int s_join(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *chan = parv[1];

        if (!u->myuser)
	{
                log("Access Denied (JOIN) to %s", u->nick);
                privmsg(u->nick, StatServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (chan)
	{
                if ( !is_channel(chan) )
		{
                        privmsg(u->nick, StatServ.nick,
                                "Please enter a valid channel");
			snprintf(errstr, BUFSIZE, "Invalid channel: %s", chan);
                        return 1;
                }

#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
                        sts(":%s SJOIN %ld %s + :@%s", settings.name, time(NULL),
                                chan, StatServ.nick);
#else
#if defined(IRCD_2_10)
                        sts(":%s NJOIN %s :@%s", settings.name, chan, StatServ.nick);
#endif
#endif

                log("%s!%s@%s made me join %s", u->nick, u->username,
                        u->hostname, chan);

                return 0;
        }
	else
	{
                privmsg(u->nick, StatServ.nick, "Please enter a valid channel");
		strcpy(errstr, "NULL channel attempt in JOIN");
                return 1;
        }
}

int s_cping(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);
	char *command = parv[1];

        if (!command)
                return 1;
        sts(":%s NOTICE %s :\001PING %s", StatServ.nick, u->nick, command);
        log("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

int s_cversion(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        sts(":%s NOTICE %s :\001VERSION %s(%s) %s %s -> %s %s\001", StatServ.nick,
             u->nick, PATCHLEVEL, SERIALNUM, settings.name, VersionFlags(),
	     version_date, version_time);
        log("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);

	return 0;
}

int s_update(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        if (!u->myuser)
	{
                log("Access Denied (UPDATE) to %s", u->nick);
                privmsg(u->nick, StatServ.nick, "Access Denied.");
		/* Not a code error */
                return 0;
        }

        if (!IsHtml())
	{
                privmsg(u->nick, StatServ.nick, "HTML Update Disabled");
                return 0;
        }

        DumpHTML();
        privmsg(u->nick, StatServ.nick, "HTML Update Forced");

        log("%s!%s@%s forces an HTML Update", u->nick, u->username,
		u->hostname);
	return 0;
}

int s_url(char *origin, int parc, char *parv[])
{
	User *u = finduser(origin);

        if (!IsHtml())
	{
                privmsg(u->nick, StatServ.nick, "HTML Dump Disabled");
                return 0;
        }

        privmsg(u->nick, StatServ.nick, "HTML Status URL: %s", settings.url);

        log("%s!%s@%s Looks up URL", u->nick, u->username, u->hostname);

	return 0;
}

#endif
