/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: timer.c,v 1.21 2001/09/12 15:27:19 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

static time_t last_spam;
static time_t last_stats_save;
#ifdef DIE_ON_MASS_COLLIDE
static time_t last_collide;
#endif
#ifdef LAG_CHECK
static time_t last_lag_check;
#endif
static time_t last_html;
static int midnight = 0;

#ifdef SPLITSERV
static time_t l_text;
#endif

#ifdef LAG_CHECK
static void TimerLagCheck();
#endif

/* Timer check, runs every time it gets a message from the server
** BEFORE it parses the message.  This will run at LEAST every time
** that the uplink sends a PING message (look in the Y line, ping 
** timings (2nd field, after the class))
**
** This function is one that I'm sure is a bottleneck...
** It gets called every time that it receives a message from the server...
** That means unless the ircd supports user mode +d and the service is set
** user mode +d, it will call chk EVERY time someone sends a JOIN/PART/NOTICE
** or a PRIVMSG to the channel.  
*/
void chk()
{
	time_t current = time(NULL);

#ifdef SPAMSERV
	if (current - last_spam > SPAMCHANGETIME)
		TimerSpam();
#endif
	if (current - last_stats_save > STATSSAVETIME)
	{
		SaveStats();
		last_stats_save = time(NULL);
	}
#ifdef DIE_ON_MASS_COLLIDE
	if (current - last_collide > COLLIDETIME)
	{
		netinfo.collides = 0;
		last_collide = time(NULL);
	}
#endif
#ifdef LAG_CHECK
	if (current - ping.last_sent > PINGERTIME)
	{
		TimerPings();
		ping.last_sent = time(NULL);
	}
	if (current - last_lag_check > LAGCHECKTIME)
	{
		TimerLagCheck();
		last_lag_check = time(NULL);
	}
#endif
	if (IsHtml() && current - last_html > settings.html_time)
	{
		DumpHTML();
		last_html = time(NULL);
	}

	if (is_midnight() == 1 && midnight == 0)
	{
		TimerMidnight();
		midnight = 1;
	}
	else
	{
		if (midnight == 1 && is_midnight() == 0)
			midnight = 0;
	}
#ifdef SPLITSERV
        if (is_delayed && (current > notice))
	{
                is_delayed = 0;
	}
        if (IsSPEnableText() && ((current - l_text) > sp.text_time))
	{
                dump_map();
                l_text = current;
	}
#endif
}

/* Reset the timers, called when the program is started
** Note the 30 second delay in last_html.  That enables 
** the service to sync at least partly before dumping out
** an HTML file, therefore making sure that you have at least
** part of your networks statistics to dump to HTML.  Without
** this delay, the dump that's created when you start the
** service only contains the stats service statistics
*/
void TimerReset()
{
	time_t current = time(NULL);
	last_spam = current;
	last_stats_save = current;
#ifdef LAG_CHECK
	last_lag_check = current;
#endif
	last_html = current - settings.html_time + HTMLDELAY;
}

#ifdef SPAMSERV
/* Create a random nick and change the spam detecter to that nick
** There should be better checking here to see if that nick already
** exists or not.
*/
void TimerSpam()
{
	char c;
	int i;
	char t[MAXNICK + 1];
	User *u = finduser(s_Spam);
	User *u2 = NULL;

	last_spam = time(NULL);

	strncpy(t, s_Spam, MAXNICK);

    while(TRUE)
    {
	memset(s_Spam, '\0', sizeof(s_Spam));
	for (i = 0; i < sizeof(s_Spam-1); i++)
	{
		c = (((u_short) random()) % 26) + 97;
		s_Spam[i] = c;
	}
	u2 = finduser(s_Spam);
	if (!u2)
		break;
    }
	if (servsock > 0) {
#if defined(IRCD_HYBRID) || defined(IRCD_HYB6) || defined(IRCD_HYB7) || defined(IRCD_OPN)
 	        sts(":%s NICK %s :%ld", t, s_Spam, time(NULL));
#else
		sts(":%s NICK %s", t, s_Spam);
#endif
		Change_User(u, s_Spam);
	}
}
#endif

#ifdef LAG_CHECK
/* Send out the lag detecting PING's */
void TimerPings()
{
	int i;
	Server *s;

	if (IsDebug())
	{
		log("Sending pings...");
	}
	ping.ulag = 0;

	for (i = 0; i < S_TABLE_SIZE; i++)
	{
		for (s = serverlist[i]; s; s = s->next)
		{
			if (!strcasecmp(settings.name, s->name))
			{
				s->ping = 0;
				continue;
			}
			sts(":%s PING %s :%s", settings.name, settings.name, s->name);
		}
	}
}

/* Announce lagged servers */
static void TimerLagCheck()
{
	Server *s;
	int i;

	if (!IsLag())
		return;

	/* Do a check to see if our uplink is lagged...
	** If it is, get out of here
	*/
	if (me.s)
	{
	    if (me.s->ping > settings.lag_time)
		return;
	}

	for (i = 0; i < S_TABLE_SIZE; i++)
	{
		for (s = serverlist[i]; s; s = s->next)
		{
			if (s->ping > settings.lag_time 
			    && time(NULL) - s->last_announce > 30 )
			{
				wallops(settings.name,
					"The current network lag for server %s"
					" is \2%d\2 seconds", s->name, s->ping);
				s->last_announce = time(NULL);
			}
		}
	}
}
#endif /* LAG_CHECK */

/* Process the Midnight rolling over */
void TimerMidnight()
{
	if (IsDebug())
	{
		log("Its midnight!!! -> %s", sctime(time(NULL)));
	}
	/* Reset Daily Statistics */
	daily.servers = netinfo.servers;
	daily.t_servers = time(NULL);
	daily.users = netinfo.users;
	daily.t_users = time(NULL);
	daily.opers = netinfo.opers;
	daily.t_opers = time(NULL);
	/* end */
	ResetLogs();
}

/* Is it Midnight (or somewhere close since chk isn't exact) */
int is_midnight()
{
	time_t current = time(NULL);
	struct tm *ltm = localtime(&current);

	if (ltm->tm_hour == 0)
		return 1;

	return 0;	
}
