/* tld.c - Stats TLD managment functions
**
** $Id: tld.c,v 1.2 2001/09/03 22:48:01 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

TLD *tldlist[T_TABLE_SIZE];

static void add_tld_to_hash_table(char *, TLD*);

/* "net" -- not DOT"net" */
TLD *findtld(char *tld)
{
	TLD *t;

	t = tldlist[HASH((unsigned char *) tld, T_TABLE_SIZE)];
	while (t && (strcasecmp(tld, t->tld) != 0))
		t = t->next;

	return t;
}

/* Add the TLD entry to a user record */
TLD *AddTLD(User *u)
{
	TLD *t = NULL;
	char *m;

	m = strrchr(u->hostname, '.');

	if (!m)
		t = findtld("num");
	else
		m++;

	/* I think this is right...for hostnames such as "Internet." */
	if (!m)
	{
		log("NULL TLD from %s (%s)", u->nick, u->hostname);
		t = findtld("unknown");
	}

	if (!t)
	{
		if (!isdigit((int) *m))
			t = findtld(m);
		else
			t = findtld("num");
	}

	if (!t)
	{
#ifndef IGNORE_ILLEGAL_TLD
		log("Unable to find TLD entry for %s (%s)", u->nick, m);
#endif
		t = findtld("unknown");
	}
	t->users++;

	return t;
}

/* Load the TLD file */
void LoadTLD()
{
        FILE *fp;
        char buf[BUFSIZE];
        char *tmp = NULL;
        char *tmp2 = NULL;
        TLD *t;
	int i;

	if ((fp = fopen(TLD_FILE, "r")) == NULL)
	{
		log("Top Level Domain Statistics not loaded: file not found.");
		return;
	}

	for (i=0; i < T_TABLE_SIZE; i++)
		tldlist[i] = NULL;

    while (fgets(buf, BUFSIZE, fp))
    {
		tmp = strchr(buf, '\t');
		*tmp = '\0';
		tmp++;
		tmp2 = strchr(tmp, '\n');
		if (tmp2)
			*tmp2 = '\0';
		tmp2 = strchr(tmp, '\r');
		if (tmp2)
			*tmp2 = '\0';

		/* This should be block allocated */
		t = smalloc(sizeof(TLD));
		t->users = 0;
		strncpy(t->country, tmp, LINE);
		t->country[LINE] = '\0';
		memcpy(t->tld, buf, sizeof(t->tld));

		add_tld_to_hash_table(t->tld, t);
    }
	fclose(fp);
}

static void add_tld_to_hash_table(char *key, TLD* t)
{
	t->hash = HASH((unsigned char *) key, T_TABLE_SIZE);
	t->next = tldlist[t->hash];
	tldlist[t->hash] = t;
}

void init_tld()
{
	TLD *t;
	int i;

	for (i = 0; i < T_TABLE_SIZE; i++)
	{
		for(t = tldlist[i]; t; t = t->next)
		{
			t->users = 0;
		}
	}

}
