/* Stats - util.c - Misc. utility functions
** $Id: util.c,v 1.33 2001/09/12 15:27:20 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

/* Generic logging routine */
void log(char *fmt, ...)
{
	va_list ap;
	FILE *logfile;
	char buf[BUFSIZE], fmtime[LINE];
	time_t tmp = time(NULL);

	va_start(ap, fmt);
	vsnprintf(buf, BUFSIZE, fmt, ap);

	strftime(fmtime, LINE, TIMEFMT, localtime(&tmp));

	if ((logfile = fopen(LOGFILE, "a")) == NULL)
		return;

	if (logfile)
		fprintf(logfile, "(%s) %s\n", fmtime, buf);

	if (IsDebug())
	{
		fprintf(stderr, "(%s) %s\n", fmtime, buf);
	}

	fclose(logfile);
	va_end(ap);
}

#ifdef SPLITSERV
/* SplitServ logging routine */
void splog(char *fmt, ...)
{
        va_list ap;
        FILE *logfile;
        char buf[BUFSIZE], fmtime[LINE];
        time_t tmp = time(NULL);

        va_start(ap, fmt);
        vsnprintf(buf, BUFSIZE, fmt, ap);

        strftime(fmtime, LINE, TIMEFMT, localtime(&tmp));

        if ((logfile = fopen(SPLITLOG, "a")) == NULL)
		return;

        if (logfile)
                fprintf(logfile, "(%s) %s\n", fmtime, buf);

        if (IsDebug())
	{
                fprintf(stderr, "(%s) %s\n", fmtime, buf);
        }

        fclose(logfile);
        va_end(ap);
}
#endif

/* Rotate the logfile, called from chk() */
void ResetLogs()
{
	char tmp[25];
	time_t t = time(NULL);

	/* Fix the date stamp so it's the logs of THAT day, not the previous
	** day with today's date stamp on it
	*/
	t = t - 86400;
	/* It's MUCH easier to manipulate this than all the vars inside the
	** struct
	*/

	strftime(tmp, 25, BACKUPLOGFMT, localtime(&t));
	rename(LOGFILE, tmp);
	log("Started fresh logfile.");

#ifdef SPLITSERV
	strftime(tmp, 25, BACKUPSPLOGFMT, localtime(&t));
	rename(SPLITLOG, tmp);
	splog("Started fresh logfile.");
#endif
}

/* Our own ctime function */
char *sctime(time_t stuff)
{
	char *s, *c;

	s = ctime(&stuff);
	if ((c = strchr(s, '\n')))
		*c = '\0';

	return s;
}

/* custom ftime format */
char *sftime(time_t stuff)
{
	static char fmtime[LINE];
	struct tm *ltm = localtime(&stuff);

	if (stuff != 0)
	{
		strftime(fmtime, LINE, FTMFMT, ltm);
	}
	else
	{
		snprintf(fmtime, LINE, "No Time/Record");
	}

	return fmtime;
}

char *irc_time(time_t ts)
{
	static char fmtime[LINE];

	struct tm *ltm = localtime(&ts);

	strftime(fmtime, LINE, IRCTFMT, ltm);

	return fmtime;
}

#if defined(CRYPT) && defined(MD5)
/* Create a random MD5 based salt for password
** encryption
*/
char* make_salt()
{
	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	static char salt[13];
	int i;

	salt[0] = '$';
	salt[1] = '1';
	salt[2] = '$';
	for (i=3; i<=10; i++)
		salt[i] = saltChars[random() % 64];
	salt[11] = '$';
	salt[12] = '\0';

	return salt;
}
#endif

#if defined(CRYPT) && !defined(MD5)
/* create a random salt for password encryption 
** This only creates DES based salts...
**
** NOTE:  This code is partially derived from mkpasswd.c
**        by Nelson Minar (minar@reed.edu)
*/
char* make_salt()
{
	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	static char salt[3];

	salt[0] = saltChars[random() % 64];
	salt[1] = saltChars[random() % 64];
	salt[2] = 0;

	return salt;
}
#endif /* CRYPT */

/* tokenize splits apart a message (in our case with the origin and command
** picked off already...
*/
int tokenize(char *message, char **parv)
{
        char *pos = NULL;
        char *next;
        int count = 0;

        if (!message)
        {
                /* Something is seriously wrong...bail */
                return -1;
        }

        /* First we find out if there's a : in the message, save that string
        ** somewhere so we can set it to the last param in parv
	** Also make sure there's a space before it...if not, then we're
	** screwed
        */
	pos = message;
	while (1)
	{
        	if ((pos = strchr(pos, ':')))
        	{
			pos--;
			if (*pos != ' ')
			{
				/* There SHOULDN'T be a problem with this... */
				pos += 2;
				continue;
			}
                	*pos = '\0';
                	/* This is BAD - get rid of the space before the : */
                	pos++;
                	*pos = '\0';
                	/* VERY BAD...over now though */
                	pos++;
			break;
        	}
		else
		{
			break;
		}
	}

        /* Now we take the beginning of message and find all the spaces...
        ** set them to \0 and use 'next' to go through the string
        */

        /* Add in checks for MAXPARA */
    while (1)
    {
        if (count == MAXPARA - 1)
	{
                /* We've reached our limit */
                log("Reached the MAXPARA limit!");
                break;
        }
        next = strchr(message, ' ');
        if (!next)
	{
                parv[count] = message;
                count++;
                break;
        }
	else
	{
                *next = '\0';
		if (*message == '\0')
		{
			next++;
			message = next;
			continue;
		}
                parv[count] = message;
                next++;
                message = next;
                count++;
        }
    }

        if (pos)
        {
                parv[count] = pos;
                count++;
        }

        return count;
}

/* msgtoken splits apart a message using only spaces (ignoring any :'s)
*/
int msgtoken(char *message, char **parv)
{
        char *next;
        int count = 0;

        if (!message)
        {
                /* Something is seriously wrong...bail */
                return -1;
        }

        /* Now we take the beginning of message and find all the spaces...
        ** set them to \0 and use 'next' to go through the string
        */

        /* Add in checks for MAXPARA */
    while (1)
    {
	if (count == MAXPARA)
	{
		/* We've reached our limit */
		log("Reached the MAXPARA limit!");
		break;
	}
        next = strchr(message, ' ');
        if (!next)
	{
                parv[count] = message;
                count++;
                break;
        }
	else
	{
		*next = '\0';
                if (*message == '\0')
		{
                        next++;
                        message = next;
                        continue;
                }
                parv[count] = message;
                next++;
                message = next;
                count++;
        }
    }

        return count;
}

/* our own malloc with logging and sanity checking */
void *smalloc(size_t size)
{
        void *buf;

        if (!size)
	{
                log("smalloc(): illegal attempt to allocate 0 bytes!");
                size = 1;
        }
        buf = malloc(size);
        if (!buf)
	{
                log("smalloc(): out of memory.");
                exit(0);
        }
        return buf;
}

/* Generate a hash value into the tables */
unsigned long HASH(const unsigned char *name, int size_of_table)
{
        unsigned long h = 0, g;

        while (*name)
	{
                h = (h << 4) + *name++;
                        if ((g = (h & 0xF0000000)))
                                h ^= g >> 24;
                        h &= ~g;
        }
        return h % size_of_table;
}

/* Lowercase a string */
char *strlower(char *s)
{
        char *t = s;
        while ((*t++ = tolower(*t)))
                ;
        return s;
}

/* Get the version flags string */
char *VersionFlags()
{
	static char ver[LINE];
	char *verptr = ver;

#ifdef FNMATCH
	*verptr++ = 'F';
#endif
#ifdef SUPPORT_MSG_UH
	*verptr++ = 'U';
#endif
#ifdef SUPPORT_MSG_OPERS
	*verptr++ = 'O';
#endif
#ifdef IGNORE_ILLEGAL_TLD
	*verptr++ = 'I';
#endif
#ifdef TRACK_VERSION
	*verptr++ = 'V';
#endif
#ifdef LAG_CHECK
	*verptr++ = 'L';
#endif
#ifdef ENABLE_JUPE
	*verptr++ = 'J';
#endif
#ifdef ALL_OPER_JUPE
	*verptr++ = 'j';
#endif
#ifdef ENABLE_QUOTE
	*verptr++ = 'Q';
#endif
#ifdef DELAY_PRIVMSG
	*verptr++ = 'D';
#endif
#ifdef LARGE_NET
	*verptr++ = 'N';
#endif
#ifdef OPER_MODES
	*verptr++ = 'M';
#endif
#ifdef DIE_ON_MASS_COLLIDE
	*verptr++ = 'C';
#endif
#ifdef DISALLOW_NICK_PRIVMSG
	*verptr++ = 'u';
#endif
#ifdef SPAMSERV
	*verptr++ = 'S';
#endif
#ifdef SPLITSERV
	*verptr++ = 'P';
#endif

	if (IsDebug())
		*verptr++ = 'd';
	if (IsNoFork())
		*verptr++ = 'n';

	*verptr++ = ' ';
	*verptr = '\0';

#ifdef IRCD_HYBRID
	strncat(ver, "HYBRID", LINE);
#endif
#ifdef IRCD_HYB6
	strncat(ver, "HYB6", LINE);
#endif
#ifdef IRCD_HYB7
	strncat(ver, "HYB7", LINE);
#endif
#ifdef IRCD_OPN
	strncat(ver, "OPN", LINE);
#endif
#ifdef IRCD_2_10
	strncat(ver, "2_10", LINE);
#endif

	return ver;
}

#ifdef OPER_MODES
char opermode(User *u)
{
	if (!u)
		return '-';
	if (u->operflags & OPER_ADMIN)
		return 'A';
	if (u->operflags & OPER_GLOBAL)
		return 'O';
	if (u->operflags & OPER_LOCAL)
		return 'o';
	/* Shouldn't get here */
	return 'x';
}
#endif /* OPER_MODES */

void SigTerm(int sig)
{
        wallops(settings.name, "Received SIGTERM: Exiting");
#ifdef STATSERV
        sts(":%s QUIT :Received SIGTERM: Exiting", StatServ.nick);
#endif
	sts(":%s QUIT :Received SIGTERM: Exiting", AdminServ.nick);
#ifdef SPLITSERV
	sts(":%s QUIT :Received SIGTERM: Exiting", SplitServ.nick);
#endif
#ifdef SPAMSERV
	sts(":%s QUIT :Received SIGTERM: Exiting", s_Spam);
#endif
        sleep(1);
        close(servsock);
        SaveStats();
        log("Received SIGTERM: Exiting");
        exit(0);
}

int is_channel(char *chan)
{
        if (chan)
	{
                if ( (chan[0] == '#') || (chan[0] == '+') )
                        return TRUE;
                return FALSE;
        }

        /* Shouldn't get here */
        return FALSE;
}
